sqlglot.dialects.presto API documentation

Edit on GitHub

# sqlglot.dialects.presto

View Source

      1from __future__ import annotations
      2
      3import typing as t
      4
      5from sqlglot import exp, generator, parser, tokens, transforms
      6from sqlglot.dialects.dialect import (
      7    Dialect,
      8    NormalizationStrategy,
      9    binary_from_function,
     10    bool_xor_sql,
     11    date_trunc_to_time,
     12    datestrtodate_sql,
     13    encode_decode_sql,
     14    build_formatted_time,
     15    if_sql,
     16    left_to_substring_sql,
     17    no_ilike_sql,
     18    no_pivot_sql,
     19    no_safe_divide_sql,
     20    no_timestamp_sql,
     21    regexp_extract_sql,
     22    rename_func,
     23    right_to_substring_sql,
     24    sha256_sql,
     25    struct_extract_sql,
     26    str_position_sql,
     27    timestamptrunc_sql,
     28    timestrtotime_sql,
     29    ts_or_ds_add_cast,
     30    unit_to_str,
     31    sequence_sql,
     32)
     33from sqlglot.dialects.hive import Hive
     34from sqlglot.dialects.mysql import MySQL
     35from sqlglot.helper import apply_index_offset, seq_get
     36from sqlglot.tokens import TokenType
     37from sqlglot.transforms import unqualify_columns
     38
     39DATE_ADD_OR_SUB = t.Union[exp.DateAdd, exp.TimestampAdd, exp.DateSub]
     40
     41
     42def _explode_to_unnest_sql(self: Presto.Generator, expression: exp.Lateral) -> str:
     43    if isinstance(expression.this, exp.Explode):
     44        return self.sql(
     45            exp.Join(
     46                this=exp.Unnest(
     47                    expressions=[expression.this.this],
     48                    alias=expression.args.get("alias"),
     49                    offset=isinstance(expression.this, exp.Posexplode),
     50                ),
     51                kind="cross",
     52            )
     53        )
     54    return self.lateral_sql(expression)
     55
     56
     57def _initcap_sql(self: Presto.Generator, expression: exp.Initcap) -> str:
     58    regex = r"(\w)(\w*)"
     59    return f"REGEXP_REPLACE({self.sql(expression, 'this')}, '{regex}', x -> UPPER(x[1]) || LOWER(x[2]))"
     60
     61
     62def _no_sort_array(self: Presto.Generator, expression: exp.SortArray) -> str:
     63    if expression.args.get("asc") == exp.false():
     64        comparator = "(a, b) -> CASE WHEN a < b THEN 1 WHEN a > b THEN -1 ELSE 0 END"
     65    else:
     66        comparator = None
     67    return self.func("ARRAY_SORT", expression.this, comparator)
     68
     69
     70def _schema_sql(self: Presto.Generator, expression: exp.Schema) -> str:
     71    if isinstance(expression.parent, exp.Property):
     72        columns = ", ".join(f"'{c.name}'" for c in expression.expressions)
     73        return f"ARRAY[{columns}]"
     74
     75    if expression.parent:
     76        for schema in expression.parent.find_all(exp.Schema):
     77            column_defs = schema.find_all(exp.ColumnDef)
     78            if column_defs and isinstance(schema.parent, exp.Property):
     79                expression.expressions.extend(column_defs)
     80
     81    return self.schema_sql(expression)
     82
     83
     84def _quantile_sql(self: Presto.Generator, expression: exp.Quantile) -> str:
     85    self.unsupported("Presto does not support exact quantiles")
     86    return self.func("APPROX_PERCENTILE", expression.this, expression.args.get("quantile"))
     87
     88
     89def _str_to_time_sql(
     90    self: Presto.Generator, expression: exp.StrToDate | exp.StrToTime | exp.TsOrDsToDate
     91) -> str:
     92    return self.func("DATE_PARSE", expression.this, self.format_time(expression))
     93
     94
     95def _ts_or_ds_to_date_sql(self: Presto.Generator, expression: exp.TsOrDsToDate) -> str:
     96    time_format = self.format_time(expression)
     97    if time_format and time_format not in (Presto.TIME_FORMAT, Presto.DATE_FORMAT):
     98        return self.sql(exp.cast(_str_to_time_sql(self, expression), exp.DataType.Type.DATE))
     99    return self.sql(
    100        exp.cast(exp.cast(expression.this, exp.DataType.Type.TIMESTAMP), exp.DataType.Type.DATE)
    101    )
    102
    103
    104def _ts_or_ds_add_sql(self: Presto.Generator, expression: exp.TsOrDsAdd) -> str:
    105    expression = ts_or_ds_add_cast(expression)
    106    unit = unit_to_str(expression)
    107    return self.func("DATE_ADD", unit, expression.expression, expression.this)
    108
    109
    110def _ts_or_ds_diff_sql(self: Presto.Generator, expression: exp.TsOrDsDiff) -> str:
    111    this = exp.cast(expression.this, exp.DataType.Type.TIMESTAMP)
    112    expr = exp.cast(expression.expression, exp.DataType.Type.TIMESTAMP)
    113    unit = unit_to_str(expression)
    114    return self.func("DATE_DIFF", unit, expr, this)
    115
    116
    117def _build_approx_percentile(args: t.List) -> exp.Expression:
    118    if len(args) == 4:
    119        return exp.ApproxQuantile(
    120            this=seq_get(args, 0),
    121            weight=seq_get(args, 1),
    122            quantile=seq_get(args, 2),
    123            accuracy=seq_get(args, 3),
    124        )
    125    if len(args) == 3:
    126        return exp.ApproxQuantile(
    127            this=seq_get(args, 0), quantile=seq_get(args, 1), accuracy=seq_get(args, 2)
    128        )
    129    return exp.ApproxQuantile.from_arg_list(args)
    130
    131
    132def _build_from_unixtime(args: t.List) -> exp.Expression:
    133    if len(args) == 3:
    134        return exp.UnixToTime(
    135            this=seq_get(args, 0),
    136            hours=seq_get(args, 1),
    137            minutes=seq_get(args, 2),
    138        )
    139    if len(args) == 2:
    140        return exp.UnixToTime(this=seq_get(args, 0), zone=seq_get(args, 1))
    141
    142    return exp.UnixToTime.from_arg_list(args)
    143
    144
    145def _unnest_sequence(expression: exp.Expression) -> exp.Expression:
    146    if isinstance(expression, exp.Table):
    147        if isinstance(expression.this, exp.GenerateSeries):
    148            unnest = exp.Unnest(expressions=[expression.this])
    149
    150            if expression.alias:
    151                return exp.alias_(unnest, alias="_u", table=[expression.alias], copy=False)
    152            return unnest
    153    return expression
    154
    155
    156def _first_last_sql(self: Presto.Generator, expression: exp.Func) -> str:
    157    """
    158    Trino doesn't support FIRST / LAST as functions, but they're valid in the context
    159    of MATCH_RECOGNIZE, so we need to preserve them in that case. In all other cases
    160    they're converted into an ARBITRARY call.
    161
    162    Reference: https://trino.io/docs/current/sql/match-recognize.html#logical-navigation-functions
    163    """
    164    if isinstance(expression.find_ancestor(exp.MatchRecognize, exp.Select), exp.MatchRecognize):
    165        return self.function_fallback_sql(expression)
    166
    167    return rename_func("ARBITRARY")(self, expression)
    168
    169
    170def _unix_to_time_sql(self: Presto.Generator, expression: exp.UnixToTime) -> str:
    171    scale = expression.args.get("scale")
    172    timestamp = self.sql(expression, "this")
    173    if scale in (None, exp.UnixToTime.SECONDS):
    174        return rename_func("FROM_UNIXTIME")(self, expression)
    175
    176    return f"FROM_UNIXTIME(CAST({timestamp} AS DOUBLE) / POW(10, {scale}))"
    177
    178
    179def _jsonextract_sql(self: Presto.Generator, expression: exp.JSONExtract) -> str:
    180    is_json_extract = self.dialect.settings.get("variant_extract_is_json_extract", True)
    181
    182    # Generate JSON_EXTRACT unless the user has configured that a Snowflake / Databricks
    183    # VARIANT extract (e.g. col:x.y) should map to dot notation (i.e ROW access) in Presto/Trino
    184    if not expression.args.get("variant_extract") or is_json_extract:
    185        return self.func(
    186            "JSON_EXTRACT", expression.this, expression.expression, *expression.expressions
    187        )
    188
    189    this = self.sql(expression, "this")
    190
    191    # Convert the JSONPath extraction `JSON_EXTRACT(col, '$.x.y) to a ROW access col.x.y
    192    segments = []
    193    for path_key in expression.expression.expressions[1:]:
    194        if not isinstance(path_key, exp.JSONPathKey):
    195            # Cannot transpile subscripts, wildcards etc to dot notation
    196            self.unsupported(f"Cannot transpile JSONPath segment '{path_key}' to ROW access")
    197            continue
    198        key = path_key.this
    199        if not exp.SAFE_IDENTIFIER_RE.match(key):
    200            key = f'"{key}"'
    201        segments.append(f".{key}")
    202
    203    expr = "".join(segments)
    204
    205    return f"{this}{expr}"
    206
    207
    208def _to_int(self: Presto.Generator, expression: exp.Expression) -> exp.Expression:
    209    if not expression.type:
    210        from sqlglot.optimizer.annotate_types import annotate_types
    211
    212        annotate_types(expression, dialect=self.dialect)
    213    if expression.type and expression.type.this not in exp.DataType.INTEGER_TYPES:
    214        return exp.cast(expression, to=exp.DataType.Type.BIGINT)
    215    return expression
    216
    217
    218def _build_to_char(args: t.List) -> exp.TimeToStr:
    219    fmt = seq_get(args, 1)
    220    if isinstance(fmt, exp.Literal):
    221        # We uppercase this to match Teradata's format mapping keys
    222        fmt.set("this", fmt.this.upper())
    223
    224    # We use "teradata" on purpose here, because the time formats are different in Presto.
    225    # See https://prestodb.io/docs/current/functions/teradata.html?highlight=to_char#to_char
    226    return build_formatted_time(exp.TimeToStr, "teradata")(args)
    227
    228
    229def _date_delta_sql(
    230    name: str, negate_interval: bool = False
    231) -> t.Callable[[Presto.Generator, DATE_ADD_OR_SUB], str]:
    232    def _delta_sql(self: Presto.Generator, expression: DATE_ADD_OR_SUB) -> str:
    233        interval = _to_int(self, expression.expression)
    234        return self.func(
    235            name,
    236            unit_to_str(expression),
    237            interval * (-1) if negate_interval else interval,
    238            expression.this,
    239        )
    240
    241    return _delta_sql
    242
    243
    244class Presto(Dialect):
    245    INDEX_OFFSET = 1
    246    NULL_ORDERING = "nulls_are_last"
    247    TIME_FORMAT = MySQL.TIME_FORMAT
    248    TIME_MAPPING = MySQL.TIME_MAPPING
    249    STRICT_STRING_CONCAT = True
    250    SUPPORTS_SEMI_ANTI_JOIN = False
    251    TYPED_DIVISION = True
    252    TABLESAMPLE_SIZE_IS_PERCENT = True
    253    LOG_BASE_FIRST: t.Optional[bool] = None
    254
    255    # https://github.com/trinodb/trino/issues/17
    256    # https://github.com/trinodb/trino/issues/12289
    257    # https://github.com/prestodb/presto/issues/2863
    258    NORMALIZATION_STRATEGY = NormalizationStrategy.CASE_INSENSITIVE
    259
    260    # The result of certain math functions in Presto/Trino is of type
    261    # equal to the input type e.g: FLOOR(5.5/2) -> DECIMAL, FLOOR(5/2) -> BIGINT
    262    ANNOTATORS = {
    263        **Dialect.ANNOTATORS,
    264        exp.Floor: lambda self, e: self._annotate_by_args(e, "this"),
    265        exp.Ceil: lambda self, e: self._annotate_by_args(e, "this"),
    266        exp.Mod: lambda self, e: self._annotate_by_args(e, "this", "expression"),
    267        exp.Round: lambda self, e: self._annotate_by_args(e, "this"),
    268        exp.Sign: lambda self, e: self._annotate_by_args(e, "this"),
    269        exp.Abs: lambda self, e: self._annotate_by_args(e, "this"),
    270        exp.Rand: lambda self, e: self._annotate_by_args(e, "this")
    271        if e.this
    272        else self._set_type(e, exp.DataType.Type.DOUBLE),
    273    }
    274
    275    class Tokenizer(tokens.Tokenizer):
    276        UNICODE_STRINGS = [
    277            (prefix + q, q)
    278            for q in t.cast(t.List[str], tokens.Tokenizer.QUOTES)
    279            for prefix in ("U&", "u&")
    280        ]
    281
    282        KEYWORDS = {
    283            **tokens.Tokenizer.KEYWORDS,
    284            "START": TokenType.BEGIN,
    285            "MATCH_RECOGNIZE": TokenType.MATCH_RECOGNIZE,
    286            "ROW": TokenType.STRUCT,
    287            "IPADDRESS": TokenType.IPADDRESS,
    288            "IPPREFIX": TokenType.IPPREFIX,
    289            "TDIGEST": TokenType.TDIGEST,
    290            "HYPERLOGLOG": TokenType.HLLSKETCH,
    291        }
    292        KEYWORDS.pop("/*+")
    293        KEYWORDS.pop("QUALIFY")
    294
    295    class Parser(parser.Parser):
    296        VALUES_FOLLOWED_BY_PAREN = False
    297
    298        FUNCTIONS = {
    299            **parser.Parser.FUNCTIONS,
    300            "ARBITRARY": exp.AnyValue.from_arg_list,
    301            "APPROX_DISTINCT": exp.ApproxDistinct.from_arg_list,
    302            "APPROX_PERCENTILE": _build_approx_percentile,
    303            "BITWISE_AND": binary_from_function(exp.BitwiseAnd),
    304            "BITWISE_NOT": lambda args: exp.BitwiseNot(this=seq_get(args, 0)),
    305            "BITWISE_OR": binary_from_function(exp.BitwiseOr),
    306            "BITWISE_XOR": binary_from_function(exp.BitwiseXor),
    307            "CARDINALITY": exp.ArraySize.from_arg_list,
    308            "CONTAINS": exp.ArrayContains.from_arg_list,
    309            "DATE_ADD": lambda args: exp.DateAdd(
    310                this=seq_get(args, 2), expression=seq_get(args, 1), unit=seq_get(args, 0)
    311            ),
    312            "DATE_DIFF": lambda args: exp.DateDiff(
    313                this=seq_get(args, 2), expression=seq_get(args, 1), unit=seq_get(args, 0)
    314            ),
    315            "DATE_FORMAT": build_formatted_time(exp.TimeToStr, "presto"),
    316            "DATE_PARSE": build_formatted_time(exp.StrToTime, "presto"),
    317            "DATE_TRUNC": date_trunc_to_time,
    318            "ELEMENT_AT": lambda args: exp.Bracket(
    319                this=seq_get(args, 0), expressions=[seq_get(args, 1)], offset=1, safe=True
    320            ),
    321            "FROM_HEX": exp.Unhex.from_arg_list,
    322            "FROM_UNIXTIME": _build_from_unixtime,
    323            "FROM_UTF8": lambda args: exp.Decode(
    324                this=seq_get(args, 0), replace=seq_get(args, 1), charset=exp.Literal.string("utf-8")
    325            ),
    326            "NOW": exp.CurrentTimestamp.from_arg_list,
    327            "REGEXP_EXTRACT": lambda args: exp.RegexpExtract(
    328                this=seq_get(args, 0), expression=seq_get(args, 1), group=seq_get(args, 2)
    329            ),
    330            "REGEXP_REPLACE": lambda args: exp.RegexpReplace(
    331                this=seq_get(args, 0),
    332                expression=seq_get(args, 1),
    333                replacement=seq_get(args, 2) or exp.Literal.string(""),
    334            ),
    335            "ROW": exp.Struct.from_arg_list,
    336            "SEQUENCE": exp.GenerateSeries.from_arg_list,
    337            "SET_AGG": exp.ArrayUniqueAgg.from_arg_list,
    338            "SPLIT_TO_MAP": exp.StrToMap.from_arg_list,
    339            "STRPOS": lambda args: exp.StrPosition(
    340                this=seq_get(args, 0), substr=seq_get(args, 1), instance=seq_get(args, 2)
    341            ),
    342            "TO_CHAR": _build_to_char,
    343            "TO_UNIXTIME": exp.TimeToUnix.from_arg_list,
    344            "TO_UTF8": lambda args: exp.Encode(
    345                this=seq_get(args, 0), charset=exp.Literal.string("utf-8")
    346            ),
    347            "MD5": exp.MD5Digest.from_arg_list,
    348            "SHA256": lambda args: exp.SHA2(this=seq_get(args, 0), length=exp.Literal.number(256)),
    349            "SHA512": lambda args: exp.SHA2(this=seq_get(args, 0), length=exp.Literal.number(512)),
    350        }
    351
    352        FUNCTION_PARSERS = parser.Parser.FUNCTION_PARSERS.copy()
    353        FUNCTION_PARSERS.pop("TRIM")
    354
    355    class Generator(generator.Generator):
    356        INTERVAL_ALLOWS_PLURAL_FORM = False
    357        JOIN_HINTS = False
    358        TABLE_HINTS = False
    359        QUERY_HINTS = False
    360        IS_BOOL_ALLOWED = False
    361        TZ_TO_WITH_TIME_ZONE = True
    362        NVL2_SUPPORTED = False
    363        STRUCT_DELIMITER = ("(", ")")
    364        LIMIT_ONLY_LITERALS = True
    365        SUPPORTS_SINGLE_ARG_CONCAT = False
    366        LIKE_PROPERTY_INSIDE_SCHEMA = True
    367        MULTI_ARG_DISTINCT = False
    368        SUPPORTS_TO_NUMBER = False
    369        HEX_FUNC = "TO_HEX"
    370        PARSE_JSON_NAME = "JSON_PARSE"
    371        PAD_FILL_PATTERN_IS_REQUIRED = True
    372
    373        PROPERTIES_LOCATION = {
    374            **generator.Generator.PROPERTIES_LOCATION,
    375            exp.LocationProperty: exp.Properties.Location.UNSUPPORTED,
    376            exp.VolatileProperty: exp.Properties.Location.UNSUPPORTED,
    377        }
    378
    379        TYPE_MAPPING = {
    380            **generator.Generator.TYPE_MAPPING,
    381            exp.DataType.Type.INT: "INTEGER",
    382            exp.DataType.Type.FLOAT: "REAL",
    383            exp.DataType.Type.BINARY: "VARBINARY",
    384            exp.DataType.Type.TEXT: "VARCHAR",
    385            exp.DataType.Type.TIMETZ: "TIME",
    386            exp.DataType.Type.TIMESTAMPTZ: "TIMESTAMP",
    387            exp.DataType.Type.STRUCT: "ROW",
    388            exp.DataType.Type.DATETIME: "TIMESTAMP",
    389            exp.DataType.Type.DATETIME64: "TIMESTAMP",
    390            exp.DataType.Type.HLLSKETCH: "HYPERLOGLOG",
    391        }
    392
    393        TRANSFORMS = {
    394            **generator.Generator.TRANSFORMS,
    395            exp.AnyValue: rename_func("ARBITRARY"),
    396            exp.ApproxQuantile: rename_func("APPROX_PERCENTILE"),
    397            exp.ArgMax: rename_func("MAX_BY"),
    398            exp.ArgMin: rename_func("MIN_BY"),
    399            exp.Array: lambda self, e: f"ARRAY[{self.expressions(e, flat=True)}]",
    400            exp.ArrayAny: rename_func("ANY_MATCH"),
    401            exp.ArrayConcat: rename_func("CONCAT"),
    402            exp.ArrayContains: rename_func("CONTAINS"),
    403            exp.ArraySize: rename_func("CARDINALITY"),
    404            exp.ArrayToString: rename_func("ARRAY_JOIN"),
    405            exp.ArrayUniqueAgg: rename_func("SET_AGG"),
    406            exp.AtTimeZone: rename_func("AT_TIMEZONE"),
    407            exp.BitwiseAnd: lambda self, e: self.func("BITWISE_AND", e.this, e.expression),
    408            exp.BitwiseLeftShift: lambda self, e: self.func(
    409                "BITWISE_ARITHMETIC_SHIFT_LEFT", e.this, e.expression
    410            ),
    411            exp.BitwiseNot: lambda self, e: self.func("BITWISE_NOT", e.this),
    412            exp.BitwiseOr: lambda self, e: self.func("BITWISE_OR", e.this, e.expression),
    413            exp.BitwiseRightShift: lambda self, e: self.func(
    414                "BITWISE_ARITHMETIC_SHIFT_RIGHT", e.this, e.expression
    415            ),
    416            exp.BitwiseXor: lambda self, e: self.func("BITWISE_XOR", e.this, e.expression),
    417            exp.Cast: transforms.preprocess([transforms.epoch_cast_to_ts]),
    418            exp.CurrentTimestamp: lambda *_: "CURRENT_TIMESTAMP",
    419            exp.DateAdd: _date_delta_sql("DATE_ADD"),
    420            exp.DateDiff: lambda self, e: self.func(
    421                "DATE_DIFF", unit_to_str(e), e.expression, e.this
    422            ),
    423            exp.DateStrToDate: datestrtodate_sql,
    424            exp.DateToDi: lambda self,
    425            e: f"CAST(DATE_FORMAT({self.sql(e, 'this')}, {Presto.DATEINT_FORMAT}) AS INT)",
    426            exp.DateSub: _date_delta_sql("DATE_ADD", negate_interval=True),
    427            exp.Decode: lambda self, e: encode_decode_sql(self, e, "FROM_UTF8"),
    428            exp.DiToDate: lambda self,
    429            e: f"CAST(DATE_PARSE(CAST({self.sql(e, 'this')} AS VARCHAR), {Presto.DATEINT_FORMAT}) AS DATE)",
    430            exp.Encode: lambda self, e: encode_decode_sql(self, e, "TO_UTF8"),
    431            exp.FileFormatProperty: lambda self, e: f"FORMAT='{e.name.upper()}'",
    432            exp.First: _first_last_sql,
    433            exp.FirstValue: _first_last_sql,
    434            exp.FromTimeZone: lambda self,
    435            e: f"WITH_TIMEZONE({self.sql(e, 'this')}, {self.sql(e, 'zone')}) AT TIME ZONE 'UTC'",
    436            exp.GenerateSeries: sequence_sql,
    437            exp.Group: transforms.preprocess([transforms.unalias_group]),
    438            exp.GroupConcat: lambda self, e: self.func(
    439                "ARRAY_JOIN", self.func("ARRAY_AGG", e.this), e.args.get("separator")
    440            ),
    441            exp.If: if_sql(),
    442            exp.ILike: no_ilike_sql,
    443            exp.Initcap: _initcap_sql,
    444            exp.JSONExtract: _jsonextract_sql,
    445            exp.Last: _first_last_sql,
    446            exp.LastValue: _first_last_sql,
    447            exp.LastDay: lambda self, e: self.func("LAST_DAY_OF_MONTH", e.this),
    448            exp.Lateral: _explode_to_unnest_sql,
    449            exp.Left: left_to_substring_sql,
    450            exp.Levenshtein: rename_func("LEVENSHTEIN_DISTANCE"),
    451            exp.LogicalAnd: rename_func("BOOL_AND"),
    452            exp.LogicalOr: rename_func("BOOL_OR"),
    453            exp.Pivot: no_pivot_sql,
    454            exp.Quantile: _quantile_sql,
    455            exp.RegexpExtract: regexp_extract_sql,
    456            exp.Right: right_to_substring_sql,
    457            exp.SafeDivide: no_safe_divide_sql,
    458            exp.Schema: _schema_sql,
    459            exp.SchemaCommentProperty: lambda self, e: self.naked_property(e),
    460            exp.Select: transforms.preprocess(
    461                [
    462                    transforms.eliminate_qualify,
    463                    transforms.eliminate_distinct_on,
    464                    transforms.explode_to_unnest(1),
    465                    transforms.eliminate_semi_and_anti_joins,
    466                ]
    467            ),
    468            exp.SortArray: _no_sort_array,
    469            exp.StrPosition: lambda self, e: str_position_sql(self, e, generate_instance=True),
    470            exp.StrToDate: lambda self, e: f"CAST({_str_to_time_sql(self, e)} AS DATE)",
    471            exp.StrToMap: rename_func("SPLIT_TO_MAP"),
    472            exp.StrToTime: _str_to_time_sql,
    473            exp.StructExtract: struct_extract_sql,
    474            exp.Table: transforms.preprocess([_unnest_sequence]),
    475            exp.Timestamp: no_timestamp_sql,
    476            exp.TimestampAdd: _date_delta_sql("DATE_ADD"),
    477            exp.TimestampTrunc: timestamptrunc_sql(),
    478            exp.TimeStrToDate: timestrtotime_sql,
    479            exp.TimeStrToTime: timestrtotime_sql,
    480            exp.TimeStrToUnix: lambda self, e: self.func(
    481                "TO_UNIXTIME", self.func("DATE_PARSE", e.this, Presto.TIME_FORMAT)
    482            ),
    483            exp.TimeToStr: lambda self, e: self.func("DATE_FORMAT", e.this, self.format_time(e)),
    484            exp.TimeToUnix: rename_func("TO_UNIXTIME"),
    485            exp.ToChar: lambda self, e: self.func("DATE_FORMAT", e.this, self.format_time(e)),
    486            exp.TryCast: transforms.preprocess([transforms.epoch_cast_to_ts]),
    487            exp.TsOrDiToDi: lambda self,
    488            e: f"CAST(SUBSTR(REPLACE(CAST({self.sql(e, 'this')} AS VARCHAR), '-', ''), 1, 8) AS INT)",
    489            exp.TsOrDsAdd: _ts_or_ds_add_sql,
    490            exp.TsOrDsDiff: _ts_or_ds_diff_sql,
    491            exp.TsOrDsToDate: _ts_or_ds_to_date_sql,
    492            exp.Unhex: rename_func("FROM_HEX"),
    493            exp.UnixToStr: lambda self,
    494            e: f"DATE_FORMAT(FROM_UNIXTIME({self.sql(e, 'this')}), {self.format_time(e)})",
    495            exp.UnixToTime: _unix_to_time_sql,
    496            exp.UnixToTimeStr: lambda self,
    497            e: f"CAST(FROM_UNIXTIME({self.sql(e, 'this')}) AS VARCHAR)",
    498            exp.VariancePop: rename_func("VAR_POP"),
    499            exp.With: transforms.preprocess([transforms.add_recursive_cte_column_names]),
    500            exp.WithinGroup: transforms.preprocess(
    501                [transforms.remove_within_group_for_percentiles]
    502            ),
    503            exp.Xor: bool_xor_sql,
    504            exp.MD5Digest: rename_func("MD5"),
    505            exp.SHA: rename_func("SHA1"),
    506            exp.SHA2: sha256_sql,
    507        }
    508
    509        RESERVED_KEYWORDS = {
    510            "alter",
    511            "and",
    512            "as",
    513            "between",
    514            "by",
    515            "case",
    516            "cast",
    517            "constraint",
    518            "create",
    519            "cross",
    520            "current_time",
    521            "current_timestamp",
    522            "deallocate",
    523            "delete",
    524            "describe",
    525            "distinct",
    526            "drop",
    527            "else",
    528            "end",
    529            "escape",
    530            "except",
    531            "execute",
    532            "exists",
    533            "extract",
    534            "false",
    535            "for",
    536            "from",
    537            "full",
    538            "group",
    539            "having",
    540            "in",
    541            "inner",
    542            "insert",
    543            "intersect",
    544            "into",
    545            "is",
    546            "join",
    547            "left",
    548            "like",
    549            "natural",
    550            "not",
    551            "null",
    552            "on",
    553            "or",
    554            "order",
    555            "outer",
    556            "prepare",
    557            "right",
    558            "select",
    559            "table",
    560            "then",
    561            "true",
    562            "union",
    563            "using",
    564            "values",
    565            "when",
    566            "where",
    567            "with",
    568        }
    569
    570        def md5_sql(self, expression: exp.MD5) -> str:
    571            this = expression.this
    572
    573            if not this.type:
    574                from sqlglot.optimizer.annotate_types import annotate_types
    575
    576                this = annotate_types(this)
    577
    578            if this.is_type(*exp.DataType.TEXT_TYPES):
    579                this = exp.Encode(this=this, charset=exp.Literal.string("utf-8"))
    580
    581            return self.func("LOWER", self.func("TO_HEX", self.func("MD5", self.sql(this))))
    582
    583        def strtounix_sql(self, expression: exp.StrToUnix) -> str:
    584            # Since `TO_UNIXTIME` requires a `TIMESTAMP`, we need to parse the argument into one.
    585            # To do this, we first try to `DATE_PARSE` it, but since this can fail when there's a
    586            # timezone involved, we wrap it in a `TRY` call and use `PARSE_DATETIME` as a fallback,
    587            # which seems to be using the same time mapping as Hive, as per:
    588            # https://joda-time.sourceforge.net/apidocs/org/joda/time/format/DateTimeFormat.html
    589            this = expression.this
    590            value_as_text = exp.cast(this, exp.DataType.Type.TEXT)
    591            value_as_timestamp = (
    592                exp.cast(this, exp.DataType.Type.TIMESTAMP) if this.is_string else this
    593            )
    594
    595            parse_without_tz = self.func("DATE_PARSE", value_as_text, self.format_time(expression))
    596
    597            formatted_value = self.func(
    598                "DATE_FORMAT", value_as_timestamp, self.format_time(expression)
    599            )
    600            parse_with_tz = self.func(
    601                "PARSE_DATETIME",
    602                formatted_value,
    603                self.format_time(expression, Hive.INVERSE_TIME_MAPPING, Hive.INVERSE_TIME_TRIE),
    604            )
    605            coalesced = self.func("COALESCE", self.func("TRY", parse_without_tz), parse_with_tz)
    606            return self.func("TO_UNIXTIME", coalesced)
    607
    608        def bracket_sql(self, expression: exp.Bracket) -> str:
    609            if expression.args.get("safe"):
    610                return self.func(
    611                    "ELEMENT_AT",
    612                    expression.this,
    613                    seq_get(
    614                        apply_index_offset(
    615                            expression.this,
    616                            expression.expressions,
    617                            1 - expression.args.get("offset", 0),
    618                        ),
    619                        0,
    620                    ),
    621                )
    622            return super().bracket_sql(expression)
    623
    624        def struct_sql(self, expression: exp.Struct) -> str:
    625            from sqlglot.optimizer.annotate_types import annotate_types
    626
    627            expression = annotate_types(expression)
    628            values: t.List[str] = []
    629            schema: t.List[str] = []
    630            unknown_type = False
    631
    632            for e in expression.expressions:
    633                if isinstance(e, exp.PropertyEQ):
    634                    if e.type and e.type.is_type(exp.DataType.Type.UNKNOWN):
    635                        unknown_type = True
    636                    else:
    637                        schema.append(f"{self.sql(e, 'this')} {self.sql(e.type)}")
    638                    values.append(self.sql(e, "expression"))
    639                else:
    640                    values.append(self.sql(e))
    641
    642            size = len(expression.expressions)
    643
    644            if not size or len(schema) != size:
    645                if unknown_type:
    646                    self.unsupported(
    647                        "Cannot convert untyped key-value definitions (try annotate_types)."
    648                    )
    649                return self.func("ROW", *values)
    650            return f"CAST(ROW({', '.join(values)}) AS ROW({', '.join(schema)}))"
    651
    652        def interval_sql(self, expression: exp.Interval) -> str:
    653            if expression.this and expression.text("unit").upper().startswith("WEEK"):
    654                return f"({expression.this.name} * INTERVAL '7' DAY)"
    655            return super().interval_sql(expression)
    656
    657        def transaction_sql(self, expression: exp.Transaction) -> str:
    658            modes = expression.args.get("modes")
    659            modes = f" {', '.join(modes)}" if modes else ""
    660            return f"START TRANSACTION{modes}"
    661
    662        def offset_limit_modifiers(
    663            self, expression: exp.Expression, fetch: bool, limit: t.Optional[exp.Fetch | exp.Limit]
    664        ) -> t.List[str]:
    665            return [
    666                self.sql(expression, "offset"),
    667                self.sql(limit),
    668            ]
    669
    670        def create_sql(self, expression: exp.Create) -> str:
    671            """
    672            Presto doesn't support CREATE VIEW with expressions (ex: `CREATE VIEW x (cola)` then `(cola)` is the expression),
    673            so we need to remove them
    674            """
    675            kind = expression.args["kind"]
    676            schema = expression.this
    677            if kind == "VIEW" and schema.expressions:
    678                expression.this.set("expressions", None)
    679            return super().create_sql(expression)
    680
    681        def delete_sql(self, expression: exp.Delete) -> str:
    682            """
    683            Presto only supports DELETE FROM for a single table without an alias, so we need
    684            to remove the unnecessary parts. If the original DELETE statement contains more
    685            than one table to be deleted, we can't safely map it 1-1 to a Presto statement.
    686            """
    687            tables = expression.args.get("tables") or [expression.this]
    688            if len(tables) > 1:
    689                return super().delete_sql(expression)
    690
    691            table = tables[0]
    692            expression.set("this", table)
    693            expression.set("tables", None)
    694
    695            if isinstance(table, exp.Table):
    696                table_alias = table.args.get("alias")
    697                if table_alias:
    698                    table_alias.pop()
    699                    expression = t.cast(exp.Delete, expression.transform(unqualify_columns))
    700
    701            return super().delete_sql(expression)

DATE_ADD_OR_SUB = typing.Union[sqlglot.expressions.DateAdd, sqlglot.expressions.TimestampAdd, sqlglot.expressions.DateSub]
class Presto(sqlglot.dialects.dialect.Dialect): View Source

    245class Presto(Dialect):
    246    INDEX_OFFSET = 1
    247    NULL_ORDERING = "nulls_are_last"
    248    TIME_FORMAT = MySQL.TIME_FORMAT
    249    TIME_MAPPING = MySQL.TIME_MAPPING
    250    STRICT_STRING_CONCAT = True
    251    SUPPORTS_SEMI_ANTI_JOIN = False
    252    TYPED_DIVISION = True
    253    TABLESAMPLE_SIZE_IS_PERCENT = True
    254    LOG_BASE_FIRST: t.Optional[bool] = None
    255
    256    # https://github.com/trinodb/trino/issues/17
    257    # https://github.com/trinodb/trino/issues/12289
    258    # https://github.com/prestodb/presto/issues/2863
    259    NORMALIZATION_STRATEGY = NormalizationStrategy.CASE_INSENSITIVE
    260
    261    # The result of certain math functions in Presto/Trino is of type
    262    # equal to the input type e.g: FLOOR(5.5/2) -> DECIMAL, FLOOR(5/2) -> BIGINT
    263    ANNOTATORS = {
    264        **Dialect.ANNOTATORS,
    265        exp.Floor: lambda self, e: self._annotate_by_args(e, "this"),
    266        exp.Ceil: lambda self, e: self._annotate_by_args(e, "this"),
    267        exp.Mod: lambda self, e: self._annotate_by_args(e, "this", "expression"),
    268        exp.Round: lambda self, e: self._annotate_by_args(e, "this"),
    269        exp.Sign: lambda self, e: self._annotate_by_args(e, "this"),
    270        exp.Abs: lambda self, e: self._annotate_by_args(e, "this"),
    271        exp.Rand: lambda self, e: self._annotate_by_args(e, "this")
    272        if e.this
    273        else self._set_type(e, exp.DataType.Type.DOUBLE),
    274    }
    275
    276    class Tokenizer(tokens.Tokenizer):
    277        UNICODE_STRINGS = [
    278            (prefix + q, q)
    279            for q in t.cast(t.List[str], tokens.Tokenizer.QUOTES)
    280            for prefix in ("U&", "u&")
    281        ]
    282
    283        KEYWORDS = {
    284            **tokens.Tokenizer.KEYWORDS,
    285            "START": TokenType.BEGIN,
    286            "MATCH_RECOGNIZE": TokenType.MATCH_RECOGNIZE,
    287            "ROW": TokenType.STRUCT,
    288            "IPADDRESS": TokenType.IPADDRESS,
    289            "IPPREFIX": TokenType.IPPREFIX,
    290            "TDIGEST": TokenType.TDIGEST,
    291            "HYPERLOGLOG": TokenType.HLLSKETCH,
    292        }
    293        KEYWORDS.pop("/*+")
    294        KEYWORDS.pop("QUALIFY")
    295
    296    class Parser(parser.Parser):
    297        VALUES_FOLLOWED_BY_PAREN = False
    298
    299        FUNCTIONS = {
    300            **parser.Parser.FUNCTIONS,
    301            "ARBITRARY": exp.AnyValue.from_arg_list,
    302            "APPROX_DISTINCT": exp.ApproxDistinct.from_arg_list,
    303            "APPROX_PERCENTILE": _build_approx_percentile,
    304            "BITWISE_AND": binary_from_function(exp.BitwiseAnd),
    305            "BITWISE_NOT": lambda args: exp.BitwiseNot(this=seq_get(args, 0)),
    306            "BITWISE_OR": binary_from_function(exp.BitwiseOr),
    307            "BITWISE_XOR": binary_from_function(exp.BitwiseXor),
    308            "CARDINALITY": exp.ArraySize.from_arg_list,
    309            "CONTAINS": exp.ArrayContains.from_arg_list,
    310            "DATE_ADD": lambda args: exp.DateAdd(
    311                this=seq_get(args, 2), expression=seq_get(args, 1), unit=seq_get(args, 0)
    312            ),
    313            "DATE_DIFF": lambda args: exp.DateDiff(
    314                this=seq_get(args, 2), expression=seq_get(args, 1), unit=seq_get(args, 0)
    315            ),
    316            "DATE_FORMAT": build_formatted_time(exp.TimeToStr, "presto"),
    317            "DATE_PARSE": build_formatted_time(exp.StrToTime, "presto"),
    318            "DATE_TRUNC": date_trunc_to_time,
    319            "ELEMENT_AT": lambda args: exp.Bracket(
    320                this=seq_get(args, 0), expressions=[seq_get(args, 1)], offset=1, safe=True
    321            ),
    322            "FROM_HEX": exp.Unhex.from_arg_list,
    323            "FROM_UNIXTIME": _build_from_unixtime,
    324            "FROM_UTF8": lambda args: exp.Decode(
    325                this=seq_get(args, 0), replace=seq_get(args, 1), charset=exp.Literal.string("utf-8")
    326            ),
    327            "NOW": exp.CurrentTimestamp.from_arg_list,
    328            "REGEXP_EXTRACT": lambda args: exp.RegexpExtract(
    329                this=seq_get(args, 0), expression=seq_get(args, 1), group=seq_get(args, 2)
    330            ),
    331            "REGEXP_REPLACE": lambda args: exp.RegexpReplace(
    332                this=seq_get(args, 0),
    333                expression=seq_get(args, 1),
    334                replacement=seq_get(args, 2) or exp.Literal.string(""),
    335            ),
    336            "ROW": exp.Struct.from_arg_list,
    337            "SEQUENCE": exp.GenerateSeries.from_arg_list,
    338            "SET_AGG": exp.ArrayUniqueAgg.from_arg_list,
    339            "SPLIT_TO_MAP": exp.StrToMap.from_arg_list,
    340            "STRPOS": lambda args: exp.StrPosition(
    341                this=seq_get(args, 0), substr=seq_get(args, 1), instance=seq_get(args, 2)
    342            ),
    343            "TO_CHAR": _build_to_char,
    344            "TO_UNIXTIME": exp.TimeToUnix.from_arg_list,
    345            "TO_UTF8": lambda args: exp.Encode(
    346                this=seq_get(args, 0), charset=exp.Literal.string("utf-8")
    347            ),
    348            "MD5": exp.MD5Digest.from_arg_list,
    349            "SHA256": lambda args: exp.SHA2(this=seq_get(args, 0), length=exp.Literal.number(256)),
    350            "SHA512": lambda args: exp.SHA2(this=seq_get(args, 0), length=exp.Literal.number(512)),
    351        }
    352
    353        FUNCTION_PARSERS = parser.Parser.FUNCTION_PARSERS.copy()
    354        FUNCTION_PARSERS.pop("TRIM")
    355
    356    class Generator(generator.Generator):
    357        INTERVAL_ALLOWS_PLURAL_FORM = False
    358        JOIN_HINTS = False
    359        TABLE_HINTS = False
    360        QUERY_HINTS = False
    361        IS_BOOL_ALLOWED = False
    362        TZ_TO_WITH_TIME_ZONE = True
    363        NVL2_SUPPORTED = False
    364        STRUCT_DELIMITER = ("(", ")")
    365        LIMIT_ONLY_LITERALS = True
    366        SUPPORTS_SINGLE_ARG_CONCAT = False
    367        LIKE_PROPERTY_INSIDE_SCHEMA = True
    368        MULTI_ARG_DISTINCT = False
    369        SUPPORTS_TO_NUMBER = False
    370        HEX_FUNC = "TO_HEX"
    371        PARSE_JSON_NAME = "JSON_PARSE"
    372        PAD_FILL_PATTERN_IS_REQUIRED = True
    373
    374        PROPERTIES_LOCATION = {
    375            **generator.Generator.PROPERTIES_LOCATION,
    376            exp.LocationProperty: exp.Properties.Location.UNSUPPORTED,
    377            exp.VolatileProperty: exp.Properties.Location.UNSUPPORTED,
    378        }
    379
    380        TYPE_MAPPING = {
    381            **generator.Generator.TYPE_MAPPING,
    382            exp.DataType.Type.INT: "INTEGER",
    383            exp.DataType.Type.FLOAT: "REAL",
    384            exp.DataType.Type.BINARY: "VARBINARY",
    385            exp.DataType.Type.TEXT: "VARCHAR",
    386            exp.DataType.Type.TIMETZ: "TIME",
    387            exp.DataType.Type.TIMESTAMPTZ: "TIMESTAMP",
    388            exp.DataType.Type.STRUCT: "ROW",
    389            exp.DataType.Type.DATETIME: "TIMESTAMP",
    390            exp.DataType.Type.DATETIME64: "TIMESTAMP",
    391            exp.DataType.Type.HLLSKETCH: "HYPERLOGLOG",
    392        }
    393
    394        TRANSFORMS = {
    395            **generator.Generator.TRANSFORMS,
    396            exp.AnyValue: rename_func("ARBITRARY"),
    397            exp.ApproxQuantile: rename_func("APPROX_PERCENTILE"),
    398            exp.ArgMax: rename_func("MAX_BY"),
    399            exp.ArgMin: rename_func("MIN_BY"),
    400            exp.Array: lambda self, e: f"ARRAY[{self.expressions(e, flat=True)}]",
    401            exp.ArrayAny: rename_func("ANY_MATCH"),
    402            exp.ArrayConcat: rename_func("CONCAT"),
    403            exp.ArrayContains: rename_func("CONTAINS"),
    404            exp.ArraySize: rename_func("CARDINALITY"),
    405            exp.ArrayToString: rename_func("ARRAY_JOIN"),
    406            exp.ArrayUniqueAgg: rename_func("SET_AGG"),
    407            exp.AtTimeZone: rename_func("AT_TIMEZONE"),
    408            exp.BitwiseAnd: lambda self, e: self.func("BITWISE_AND", e.this, e.expression),
    409            exp.BitwiseLeftShift: lambda self, e: self.func(
    410                "BITWISE_ARITHMETIC_SHIFT_LEFT", e.this, e.expression
    411            ),
    412            exp.BitwiseNot: lambda self, e: self.func("BITWISE_NOT", e.this),
    413            exp.BitwiseOr: lambda self, e: self.func("BITWISE_OR", e.this, e.expression),
    414            exp.BitwiseRightShift: lambda self, e: self.func(
    415                "BITWISE_ARITHMETIC_SHIFT_RIGHT", e.this, e.expression
    416            ),
    417            exp.BitwiseXor: lambda self, e: self.func("BITWISE_XOR", e.this, e.expression),
    418            exp.Cast: transforms.preprocess([transforms.epoch_cast_to_ts]),
    419            exp.CurrentTimestamp: lambda *_: "CURRENT_TIMESTAMP",
    420            exp.DateAdd: _date_delta_sql("DATE_ADD"),
    421            exp.DateDiff: lambda self, e: self.func(
    422                "DATE_DIFF", unit_to_str(e), e.expression, e.this
    423            ),
    424            exp.DateStrToDate: datestrtodate_sql,
    425            exp.DateToDi: lambda self,
    426            e: f"CAST(DATE_FORMAT({self.sql(e, 'this')}, {Presto.DATEINT_FORMAT}) AS INT)",
    427            exp.DateSub: _date_delta_sql("DATE_ADD", negate_interval=True),
    428            exp.Decode: lambda self, e: encode_decode_sql(self, e, "FROM_UTF8"),
    429            exp.DiToDate: lambda self,
    430            e: f"CAST(DATE_PARSE(CAST({self.sql(e, 'this')} AS VARCHAR), {Presto.DATEINT_FORMAT}) AS DATE)",
    431            exp.Encode: lambda self, e: encode_decode_sql(self, e, "TO_UTF8"),
    432            exp.FileFormatProperty: lambda self, e: f"FORMAT='{e.name.upper()}'",
    433            exp.First: _first_last_sql,
    434            exp.FirstValue: _first_last_sql,
    435            exp.FromTimeZone: lambda self,
    436            e: f"WITH_TIMEZONE({self.sql(e, 'this')}, {self.sql(e, 'zone')}) AT TIME ZONE 'UTC'",
    437            exp.GenerateSeries: sequence_sql,
    438            exp.Group: transforms.preprocess([transforms.unalias_group]),
    439            exp.GroupConcat: lambda self, e: self.func(
    440                "ARRAY_JOIN", self.func("ARRAY_AGG", e.this), e.args.get("separator")
    441            ),
    442            exp.If: if_sql(),
    443            exp.ILike: no_ilike_sql,
    444            exp.Initcap: _initcap_sql,
    445            exp.JSONExtract: _jsonextract_sql,
    446            exp.Last: _first_last_sql,
    447            exp.LastValue: _first_last_sql,
    448            exp.LastDay: lambda self, e: self.func("LAST_DAY_OF_MONTH", e.this),
    449            exp.Lateral: _explode_to_unnest_sql,
    450            exp.Left: left_to_substring_sql,
    451            exp.Levenshtein: rename_func("LEVENSHTEIN_DISTANCE"),
    452            exp.LogicalAnd: rename_func("BOOL_AND"),
    453            exp.LogicalOr: rename_func("BOOL_OR"),
    454            exp.Pivot: no_pivot_sql,
    455            exp.Quantile: _quantile_sql,
    456            exp.RegexpExtract: regexp_extract_sql,
    457            exp.Right: right_to_substring_sql,
    458            exp.SafeDivide: no_safe_divide_sql,
    459            exp.Schema: _schema_sql,
    460            exp.SchemaCommentProperty: lambda self, e: self.naked_property(e),
    461            exp.Select: transforms.preprocess(
    462                [
    463                    transforms.eliminate_qualify,
    464                    transforms.eliminate_distinct_on,
    465                    transforms.explode_to_unnest(1),
    466                    transforms.eliminate_semi_and_anti_joins,
    467                ]
    468            ),
    469            exp.SortArray: _no_sort_array,
    470            exp.StrPosition: lambda self, e: str_position_sql(self, e, generate_instance=True),
    471            exp.StrToDate: lambda self, e: f"CAST({_str_to_time_sql(self, e)} AS DATE)",
    472            exp.StrToMap: rename_func("SPLIT_TO_MAP"),
    473            exp.StrToTime: _str_to_time_sql,
    474            exp.StructExtract: struct_extract_sql,
    475            exp.Table: transforms.preprocess([_unnest_sequence]),
    476            exp.Timestamp: no_timestamp_sql,
    477            exp.TimestampAdd: _date_delta_sql("DATE_ADD"),
    478            exp.TimestampTrunc: timestamptrunc_sql(),
    479            exp.TimeStrToDate: timestrtotime_sql,
    480            exp.TimeStrToTime: timestrtotime_sql,
    481            exp.TimeStrToUnix: lambda self, e: self.func(
    482                "TO_UNIXTIME", self.func("DATE_PARSE", e.this, Presto.TIME_FORMAT)
    483            ),
    484            exp.TimeToStr: lambda self, e: self.func("DATE_FORMAT", e.this, self.format_time(e)),
    485            exp.TimeToUnix: rename_func("TO_UNIXTIME"),
    486            exp.ToChar: lambda self, e: self.func("DATE_FORMAT", e.this, self.format_time(e)),
    487            exp.TryCast: transforms.preprocess([transforms.epoch_cast_to_ts]),
    488            exp.TsOrDiToDi: lambda self,
    489            e: f"CAST(SUBSTR(REPLACE(CAST({self.sql(e, 'this')} AS VARCHAR), '-', ''), 1, 8) AS INT)",
    490            exp.TsOrDsAdd: _ts_or_ds_add_sql,
    491            exp.TsOrDsDiff: _ts_or_ds_diff_sql,
    492            exp.TsOrDsToDate: _ts_or_ds_to_date_sql,
    493            exp.Unhex: rename_func("FROM_HEX"),
    494            exp.UnixToStr: lambda self,
    495            e: f"DATE_FORMAT(FROM_UNIXTIME({self.sql(e, 'this')}), {self.format_time(e)})",
    496            exp.UnixToTime: _unix_to_time_sql,
    497            exp.UnixToTimeStr: lambda self,
    498            e: f"CAST(FROM_UNIXTIME({self.sql(e, 'this')}) AS VARCHAR)",
    499            exp.VariancePop: rename_func("VAR_POP"),
    500            exp.With: transforms.preprocess([transforms.add_recursive_cte_column_names]),
    501            exp.WithinGroup: transforms.preprocess(
    502                [transforms.remove_within_group_for_percentiles]
    503            ),
    504            exp.Xor: bool_xor_sql,
    505            exp.MD5Digest: rename_func("MD5"),
    506            exp.SHA: rename_func("SHA1"),
    507            exp.SHA2: sha256_sql,
    508        }
    509
    510        RESERVED_KEYWORDS = {
    511            "alter",
    512            "and",
    513            "as",
    514            "between",
    515            "by",
    516            "case",
    517            "cast",
    518            "constraint",
    519            "create",
    520            "cross",
    521            "current_time",
    522            "current_timestamp",
    523            "deallocate",
    524            "delete",
    525            "describe",
    526            "distinct",
    527            "drop",
    528            "else",
    529            "end",
    530            "escape",
    531            "except",
    532            "execute",
    533            "exists",
    534            "extract",
    535            "false",
    536            "for",
    537            "from",
    538            "full",
    539            "group",
    540            "having",
    541            "in",
    542            "inner",
    543            "insert",
    544            "intersect",
    545            "into",
    546            "is",
    547            "join",
    548            "left",
    549            "like",
    550            "natural",
    551            "not",
    552            "null",
    553            "on",
    554            "or",
    555            "order",
    556            "outer",
    557            "prepare",
    558            "right",
    559            "select",
    560            "table",
    561            "then",
    562            "true",
    563            "union",
    564            "using",
    565            "values",
    566            "when",
    567            "where",
    568            "with",
    569        }
    570
    571        def md5_sql(self, expression: exp.MD5) -> str:
    572            this = expression.this
    573
    574            if not this.type:
    575                from sqlglot.optimizer.annotate_types import annotate_types
    576
    577                this = annotate_types(this)
    578
    579            if this.is_type(*exp.DataType.TEXT_TYPES):
    580                this = exp.Encode(this=this, charset=exp.Literal.string("utf-8"))
    581
    582            return self.func("LOWER", self.func("TO_HEX", self.func("MD5", self.sql(this))))
    583
    584        def strtounix_sql(self, expression: exp.StrToUnix) -> str:
    585            # Since `TO_UNIXTIME` requires a `TIMESTAMP`, we need to parse the argument into one.
    586            # To do this, we first try to `DATE_PARSE` it, but since this can fail when there's a
    587            # timezone involved, we wrap it in a `TRY` call and use `PARSE_DATETIME` as a fallback,
    588            # which seems to be using the same time mapping as Hive, as per:
    589            # https://joda-time.sourceforge.net/apidocs/org/joda/time/format/DateTimeFormat.html
    590            this = expression.this
    591            value_as_text = exp.cast(this, exp.DataType.Type.TEXT)
    592            value_as_timestamp = (
    593                exp.cast(this, exp.DataType.Type.TIMESTAMP) if this.is_string else this
    594            )
    595
    596            parse_without_tz = self.func("DATE_PARSE", value_as_text, self.format_time(expression))
    597
    598            formatted_value = self.func(
    599                "DATE_FORMAT", value_as_timestamp, self.format_time(expression)
    600            )
    601            parse_with_tz = self.func(
    602                "PARSE_DATETIME",
    603                formatted_value,
    604                self.format_time(expression, Hive.INVERSE_TIME_MAPPING, Hive.INVERSE_TIME_TRIE),
    605            )
    606            coalesced = self.func("COALESCE", self.func("TRY", parse_without_tz), parse_with_tz)
    607            return self.func("TO_UNIXTIME", coalesced)
    608
    609        def bracket_sql(self, expression: exp.Bracket) -> str:
    610            if expression.args.get("safe"):
    611                return self.func(
    612                    "ELEMENT_AT",
    613                    expression.this,
    614                    seq_get(
    615                        apply_index_offset(
    616                            expression.this,
    617                            expression.expressions,
    618                            1 - expression.args.get("offset", 0),
    619                        ),
    620                        0,
    621                    ),
    622                )
    623            return super().bracket_sql(expression)
    624
    625        def struct_sql(self, expression: exp.Struct) -> str:
    626            from sqlglot.optimizer.annotate_types import annotate_types
    627
    628            expression = annotate_types(expression)
    629            values: t.List[str] = []
    630            schema: t.List[str] = []
    631            unknown_type = False
    632
    633            for e in expression.expressions:
    634                if isinstance(e, exp.PropertyEQ):
    635                    if e.type and e.type.is_type(exp.DataType.Type.UNKNOWN):
    636                        unknown_type = True
    637                    else:
    638                        schema.append(f"{self.sql(e, 'this')} {self.sql(e.type)}")
    639                    values.append(self.sql(e, "expression"))
    640                else:
    641                    values.append(self.sql(e))
    642
    643            size = len(expression.expressions)
    644
    645            if not size or len(schema) != size:
    646                if unknown_type:
    647                    self.unsupported(
    648                        "Cannot convert untyped key-value definitions (try annotate_types)."
    649                    )
    650                return self.func("ROW", *values)
    651            return f"CAST(ROW({', '.join(values)}) AS ROW({', '.join(schema)}))"
    652
    653        def interval_sql(self, expression: exp.Interval) -> str:
    654            if expression.this and expression.text("unit").upper().startswith("WEEK"):
    655                return f"({expression.this.name} * INTERVAL '7' DAY)"
    656            return super().interval_sql(expression)
    657
    658        def transaction_sql(self, expression: exp.Transaction) -> str:
    659            modes = expression.args.get("modes")
    660            modes = f" {', '.join(modes)}" if modes else ""
    661            return f"START TRANSACTION{modes}"
    662
    663        def offset_limit_modifiers(
    664            self, expression: exp.Expression, fetch: bool, limit: t.Optional[exp.Fetch | exp.Limit]
    665        ) -> t.List[str]:
    666            return [
    667                self.sql(expression, "offset"),
    668                self.sql(limit),
    669            ]
    670
    671        def create_sql(self, expression: exp.Create) -> str:
    672            """
    673            Presto doesn't support CREATE VIEW with expressions (ex: `CREATE VIEW x (cola)` then `(cola)` is the expression),
    674            so we need to remove them
    675            """
    676            kind = expression.args["kind"]
    677            schema = expression.this
    678            if kind == "VIEW" and schema.expressions:
    679                expression.this.set("expressions", None)
    680            return super().create_sql(expression)
    681
    682        def delete_sql(self, expression: exp.Delete) -> str:
    683            """
    684            Presto only supports DELETE FROM for a single table without an alias, so we need
    685            to remove the unnecessary parts. If the original DELETE statement contains more
    686            than one table to be deleted, we can't safely map it 1-1 to a Presto statement.
    687            """
    688            tables = expression.args.get("tables") or [expression.this]
    689            if len(tables) > 1:
    690                return super().delete_sql(expression)
    691
    692            table = tables[0]
    693            expression.set("this", table)
    694            expression.set("tables", None)
    695
    696            if isinstance(table, exp.Table):
    697                table_alias = table.args.get("alias")
    698                if table_alias:
    699                    table_alias.pop()
    700                    expression = t.cast(exp.Delete, expression.transform(unqualify_columns))
    701
    702            return super().delete_sql(expression)

INDEX_OFFSET = 1

The base index offset for arrays.

NULL_ORDERING = 'nulls_are_last'

Default `NULL` ordering method to use if not explicitly set. Possible values: `"nulls_are_small"`, `"nulls_are_large"`, `"nulls_are_last"`

TIME_FORMAT = "'%Y-%m-%d %T'"
TIME_MAPPING = {'%M': '%B', '%c': '%-m', '%e': '%-d', '%h': '%I', '%i': '%M', '%s': '%S', '%u': '%W', '%k': '%-H', '%l': '%-I', '%T': '%H:%M:%S', '%W': '%a'}

Associates this dialect's time formats with their equivalent Python `strftime` formats.

STRICT_STRING_CONCAT = True

Whether `CONCAT`'s arguments must be strings.

SUPPORTS_SEMI_ANTI_JOIN = False

Whether `SEMI` or `ANTI` joins are supported.

TYPED_DIVISION = True

Whether the behavior of `a / b` depends on the types of `a` and `b`. False means `a / b` is always float division. True means `a / b` is integer division if both `a` and `b` are integers.

TABLESAMPLE_SIZE_IS_PERCENT = True

Whether a size in the table sample clause represents percentage.

LOG_BASE_FIRST: Optional[bool] = None

Whether the base comes first in the `LOG` function. Possible values: `True`, `False`, `None` (two arguments are not supported by `LOG`)

NORMALIZATION_STRATEGY = <NormalizationStrategy.CASE_INSENSITIVE: 'CASE_INSENSITIVE'>

Specifies the strategy according to which identifiers should be normalized.

ANNOTATORS = {<class 'sqlglot.expressions.Alias'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.BitwiseNot'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.Neg'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.Not'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.Paren'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.PivotAlias'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.Unary'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.Add'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.And'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.ArrayContains'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.ArrayContainsAll'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.ArrayOverlaps'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.Binary'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.BitwiseAnd'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.BitwiseLeftShift'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.BitwiseOr'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.BitwiseRightShift'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.BitwiseXor'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.Collate'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.Connector'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.Corr'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.CovarPop'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.CovarSamp'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.DPipe'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.Distance'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.Div'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.Dot'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.EQ'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.Escape'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.GT'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.GTE'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.Glob'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.ILike'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.ILikeAny'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.IntDiv'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.Is'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.JSONArrayContains'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.JSONBContains'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.JSONBExtract'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.JSONBExtractScalar'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.JSONExtract'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.JSONExtractScalar'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.Kwarg'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.LT'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.LTE'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.Like'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.LikeAny'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.Mod'>: <function Presto.<lambda>>, <class 'sqlglot.expressions.Mul'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.NEQ'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.NullSafeEQ'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.NullSafeNEQ'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.Operator'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.Or'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.Overlaps'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.Pow'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.PropertyEQ'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.RegexpILike'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.RegexpLike'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.SimilarTo'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.Slice'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.Sub'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.Xor'>: <function Dialect.<dictcomp>.<lambda>>, <class 'sqlglot.expressions.ApproxDistinct'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.ArraySize'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.Length'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.Count'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.Between'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.Boolean'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.In'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.TimeStrToDate'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.DateStrToDate'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.CurrentDate'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.StrToDate'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.DateFromParts'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.DiToDate'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.TsOrDsToDate'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.Date'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.DatetimeAdd'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.Datetime'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.CurrentDatetime'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.DatetimeSub'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.Ln'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.Stddev'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.StddevPop'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.Sqrt'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.Exp'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.Avg'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.ApproxQuantile'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.VariancePop'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.Round'>: <function Presto.<lambda>>, <class 'sqlglot.expressions.Quantile'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.SafeDivide'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.Variance'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.StddevSamp'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.Log'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.DateToDi'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.TimestampDiff'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.Levenshtein'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.Sign'>: <function Presto.<lambda>>, <class 'sqlglot.expressions.Ceil'>: <function Presto.<lambda>>, <class 'sqlglot.expressions.TsOrDiToDi'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.StrPosition'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.DatetimeDiff'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.TimeDiff'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.DateDiff'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.ParseJSON'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.Time'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.TimeStrToTime'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.CurrentTime'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.UnixToTime'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.StrToTime'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.TimestampSub'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.TimestampAdd'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.TimeAdd'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.TimeSub'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.CurrentTimestamp'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.Week'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.Quarter'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.Month'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.Day'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.Year'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.Concat'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.UnixToStr'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.DateToDateStr'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.UnixToTimeStr'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.TimeToTimeStr'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.GroupConcat'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.Lower'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.Initcap'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.TimeToStr'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.ArrayConcat'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.TsOrDsToDateStr'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.Trim'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.Substring'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.Upper'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.ConcatWs'>: <function _annotate_with_type_lambda.<locals>.<lambda>>, <class 'sqlglot.expressions.Abs'>: <function Presto.<lambda>>, <class 'sqlglot.expressions.Anonymous'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.Array'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.ArrayAgg'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.Bracket'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.Cast'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.Case'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.Coalesce'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.DataType'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.DateAdd'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.DateSub'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.DateTrunc'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.Distinct'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.Explode'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.Extract'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.Filter'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.GenerateDateArray'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.If'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.Interval'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.Least'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.Literal'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.Map'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.Max'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.Min'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.Null'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.Nullif'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.Struct'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.Sum'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.Timestamp'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.ToMap'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.TryCast'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.Unnest'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.VarMap'>: <function Dialect.<lambda>>, <class 'sqlglot.expressions.Floor'>: <function Presto.<lambda>>, <class 'sqlglot.expressions.Rand'>: <function Presto.<lambda>>}
SUPPORTS_COLUMN_JOIN_MARKS = False

Whether the old-style outer join (+) syntax is supported.

tokenizer_class = <class 'Presto.Tokenizer'>
jsonpath_tokenizer_class = <class 'sqlglot.tokens.JSONPathTokenizer'>
parser_class = <class 'Presto.Parser'>
generator_class = <class 'Presto.Generator'>
TIME_TRIE: Dict = {'%': {'M': {0: True}, 'c': {0: True}, 'e': {0: True}, 'h': {0: True}, 'i': {0: True}, 's': {0: True}, 'u': {0: True}, 'k': {0: True}, 'l': {0: True}, 'T': {0: True}, 'W': {0: True}}}
FORMAT_TRIE: Dict = {'%': {'M': {0: True}, 'c': {0: True}, 'e': {0: True}, 'h': {0: True}, 'i': {0: True}, 's': {0: True}, 'u': {0: True}, 'k': {0: True}, 'l': {0: True}, 'T': {0: True}, 'W': {0: True}}}
INVERSE_TIME_MAPPING: Dict[str, str] = {'%B': '%M', '%-m': '%c', '%-d': '%e', '%I': '%h', '%M': '%i', '%S': '%s', '%W': '%u', '%-H': '%k', '%-I': '%l', '%H:%M:%S': '%T', '%a': '%W'}
INVERSE_TIME_TRIE: Dict = {'%': {'B': {0: True}, '-': {'m': {0: True}, 'd': {0: True}, 'H': {0: True}, 'I': {0: True}}, 'I': {0: True}, 'M': {0: True}, 'S': {0: True}, 'W': {0: True}, 'H': {':': {'%': {'M': {':': {'%': {'S': {0: True}}}}}}}, 'a': {0: True}}}
INVERSE_FORMAT_MAPPING: Dict[str, str] = {}
INVERSE_FORMAT_TRIE: Dict = {}
ESCAPED_SEQUENCES: Dict[str, str] = {}
QUOTE_START = "'"
QUOTE_END = "'"
IDENTIFIER_START = '"'
IDENTIFIER_END = '"'
BIT_START: Optional[str] = None
BIT_END: Optional[str] = None
HEX_START: Optional[str] = None
HEX_END: Optional[str] = None
BYTE_START: Optional[str] = None
BYTE_END: Optional[str] = None
UNICODE_START: Optional[str] = "U&'"
UNICODE_END: Optional[str] = "'"

##### Inherited Members

sqlglot.dialects.dialect.Dialect
  Dialect
  WEEK_OFFSET
  UNNEST_COLUMN_ONLY
  ALIAS_POST_TABLESAMPLE
  IDENTIFIERS_CAN_START_WITH_DIGIT
  DPIPE_IS_STRING_CONCAT
  SUPPORTS_USER_DEFINED_TYPES
  COPY_PARAMS_ARE_CSV
  NORMALIZE_FUNCTIONS
  SAFE_DIVISION
  CONCAT_COALESCE
  HEX_LOWERCASE
  DATE_FORMAT
  DATEINT_FORMAT
  FORMAT_MAPPING
  UNESCAPED_SEQUENCES
  PSEUDOCOLUMNS
  PREFER_CTE_ALIAS_COLUMN
  FORCE_EARLY_ALIAS_REF_EXPANSION
  EXPAND_ALIAS_REFS_EARLY_ONLY_IN_GROUP_BY
  SUPPORTS_ORDER_BY_ALL
  DATE_PART_MAPPING
  TYPE_TO_EXPRESSIONS
  get_or_raise
  format_time
  settings
  normalize_identifier
  case_sensitive
  can_identify
  quote_identifier
  to_json_path
  parse
  parse_into
  generate
  transpile
  tokenize
  tokenizer
  jsonpath_tokenizer
  parser
  generator

class Presto.Tokenizer(sqlglot.tokens.Tokenizer): View Source

    276    class Tokenizer(tokens.Tokenizer):
    277        UNICODE_STRINGS = [
    278            (prefix + q, q)
    279            for q in t.cast(t.List[str], tokens.Tokenizer.QUOTES)
    280            for prefix in ("U&", "u&")
    281        ]
    282
    283        KEYWORDS = {
    284            **tokens.Tokenizer.KEYWORDS,
    285            "START": TokenType.BEGIN,
    286            "MATCH_RECOGNIZE": TokenType.MATCH_RECOGNIZE,
    287            "ROW": TokenType.STRUCT,
    288            "IPADDRESS": TokenType.IPADDRESS,
    289            "IPPREFIX": TokenType.IPPREFIX,
    290            "TDIGEST": TokenType.TDIGEST,
    291            "HYPERLOGLOG": TokenType.HLLSKETCH,
    292        }
    293        KEYWORDS.pop("/*+")
    294        KEYWORDS.pop("QUALIFY")

UNICODE_STRINGS = [("U&'", "'"), ("u&'", "'")]
KEYWORDS = {'{%': <TokenType.BLOCK_START: 'BLOCK_START'>, '{%+': <TokenType.BLOCK_START: 'BLOCK_START'>, '{%-': <TokenType.BLOCK_START: 'BLOCK_START'>, '%}': <TokenType.BLOCK_END: 'BLOCK_END'>, '+%}': <TokenType.BLOCK_END: 'BLOCK_END'>, '-%}': <TokenType.BLOCK_END: 'BLOCK_END'>, '{{+': <TokenType.BLOCK_START: 'BLOCK_START'>, '{{-': <TokenType.BLOCK_START: 'BLOCK_START'>, '+}}': <TokenType.BLOCK_END: 'BLOCK_END'>, '-}}': <TokenType.BLOCK_END: 'BLOCK_END'>, '==': <TokenType.EQ: 'EQ'>, '::': <TokenType.DCOLON: 'DCOLON'>, '||': <TokenType.DPIPE: 'DPIPE'>, '>=': <TokenType.GTE: 'GTE'>, '<=': <TokenType.LTE: 'LTE'>, '<>': <TokenType.NEQ: 'NEQ'>, '!=': <TokenType.NEQ: 'NEQ'>, ':=': <TokenType.COLON_EQ: 'COLON_EQ'>, '<=>': <TokenType.NULLSAFE_EQ: 'NULLSAFE_EQ'>, '->': <TokenType.ARROW: 'ARROW'>, '->>': <TokenType.DARROW: 'DARROW'>, '=>': <TokenType.FARROW: 'FARROW'>, '#>': <TokenType.HASH_ARROW: 'HASH_ARROW'>, '#>>': <TokenType.DHASH_ARROW: 'DHASH_ARROW'>, '<->': <TokenType.LR_ARROW: 'LR_ARROW'>, '&&': <TokenType.DAMP: 'DAMP'>, '??': <TokenType.DQMARK: 'DQMARK'>, 'ALL': <TokenType.ALL: 'ALL'>, 'ALWAYS': <TokenType.ALWAYS: 'ALWAYS'>, 'AND': <TokenType.AND: 'AND'>, 'ANTI': <TokenType.ANTI: 'ANTI'>, 'ANY': <TokenType.ANY: 'ANY'>, 'ASC': <TokenType.ASC: 'ASC'>, 'AS': <TokenType.ALIAS: 'ALIAS'>, 'ASOF': <TokenType.ASOF: 'ASOF'>, 'AUTOINCREMENT': <TokenType.AUTO_INCREMENT: 'AUTO_INCREMENT'>, 'AUTO_INCREMENT': <TokenType.AUTO_INCREMENT: 'AUTO_INCREMENT'>, 'BEGIN': <TokenType.BEGIN: 'BEGIN'>, 'BETWEEN': <TokenType.BETWEEN: 'BETWEEN'>, 'CACHE': <TokenType.CACHE: 'CACHE'>, 'UNCACHE': <TokenType.UNCACHE: 'UNCACHE'>, 'CASE': <TokenType.CASE: 'CASE'>, 'CHARACTER SET': <TokenType.CHARACTER_SET: 'CHARACTER_SET'>, 'CLUSTER BY': <TokenType.CLUSTER_BY: 'CLUSTER_BY'>, 'COLLATE': <TokenType.COLLATE: 'COLLATE'>, 'COLUMN': <TokenType.COLUMN: 'COLUMN'>, 'COMMIT': <TokenType.COMMIT: 'COMMIT'>, 'CONNECT BY': <TokenType.CONNECT_BY: 'CONNECT_BY'>, 'CONSTRAINT': <TokenType.CONSTRAINT: 'CONSTRAINT'>, 'COPY': <TokenType.COPY: 'COPY'>, 'CREATE': <TokenType.CREATE: 'CREATE'>, 'CROSS': <TokenType.CROSS: 'CROSS'>, 'CUBE': <TokenType.CUBE: 'CUBE'>, 'CURRENT_DATE': <TokenType.CURRENT_DATE: 'CURRENT_DATE'>, 'CURRENT_TIME': <TokenType.CURRENT_TIME: 'CURRENT_TIME'>, 'CURRENT_TIMESTAMP': <TokenType.CURRENT_TIMESTAMP: 'CURRENT_TIMESTAMP'>, 'CURRENT_USER': <TokenType.CURRENT_USER: 'CURRENT_USER'>, 'DATABASE': <TokenType.DATABASE: 'DATABASE'>, 'DEFAULT': <TokenType.DEFAULT: 'DEFAULT'>, 'DELETE': <TokenType.DELETE: 'DELETE'>, 'DESC': <TokenType.DESC: 'DESC'>, 'DESCRIBE': <TokenType.DESCRIBE: 'DESCRIBE'>, 'DISTINCT': <TokenType.DISTINCT: 'DISTINCT'>, 'DISTRIBUTE BY': <TokenType.DISTRIBUTE_BY: 'DISTRIBUTE_BY'>, 'DIV': <TokenType.DIV: 'DIV'>, 'DROP': <TokenType.DROP: 'DROP'>, 'ELSE': <TokenType.ELSE: 'ELSE'>, 'END': <TokenType.END: 'END'>, 'ENUM': <TokenType.ENUM: 'ENUM'>, 'ESCAPE': <TokenType.ESCAPE: 'ESCAPE'>, 'EXCEPT': <TokenType.EXCEPT: 'EXCEPT'>, 'EXECUTE': <TokenType.EXECUTE: 'EXECUTE'>, 'EXISTS': <TokenType.EXISTS: 'EXISTS'>, 'FALSE': <TokenType.FALSE: 'FALSE'>, 'FETCH': <TokenType.FETCH: 'FETCH'>, 'FILTER': <TokenType.FILTER: 'FILTER'>, 'FIRST': <TokenType.FIRST: 'FIRST'>, 'FULL': <TokenType.FULL: 'FULL'>, 'FUNCTION': <TokenType.FUNCTION: 'FUNCTION'>, 'FOR': <TokenType.FOR: 'FOR'>, 'FOREIGN KEY': <TokenType.FOREIGN_KEY: 'FOREIGN_KEY'>, 'FORMAT': <TokenType.FORMAT: 'FORMAT'>, 'FROM': <TokenType.FROM: 'FROM'>, 'GEOGRAPHY': <TokenType.GEOGRAPHY: 'GEOGRAPHY'>, 'GEOMETRY': <TokenType.GEOMETRY: 'GEOMETRY'>, 'GLOB': <TokenType.GLOB: 'GLOB'>, 'GROUP BY': <TokenType.GROUP_BY: 'GROUP_BY'>, 'GROUPING SETS': <TokenType.GROUPING_SETS: 'GROUPING_SETS'>, 'HAVING': <TokenType.HAVING: 'HAVING'>, 'ILIKE': <TokenType.ILIKE: 'ILIKE'>, 'IN': <TokenType.IN: 'IN'>, 'INDEX': <TokenType.INDEX: 'INDEX'>, 'INET': <TokenType.INET: 'INET'>, 'INNER': <TokenType.INNER: 'INNER'>, 'INSERT': <TokenType.INSERT: 'INSERT'>, 'INTERVAL': <TokenType.INTERVAL: 'INTERVAL'>, 'INTERSECT': <TokenType.INTERSECT: 'INTERSECT'>, 'INTO': <TokenType.INTO: 'INTO'>, 'IS': <TokenType.IS: 'IS'>, 'ISNULL': <TokenType.ISNULL: 'ISNULL'>, 'JOIN': <TokenType.JOIN: 'JOIN'>, 'KEEP': <TokenType.KEEP: 'KEEP'>, 'KILL': <TokenType.KILL: 'KILL'>, 'LATERAL': <TokenType.LATERAL: 'LATERAL'>, 'LEFT': <TokenType.LEFT: 'LEFT'>, 'LIKE': <TokenType.LIKE: 'LIKE'>, 'LIMIT': <TokenType.LIMIT: 'LIMIT'>, 'LOAD': <TokenType.LOAD: 'LOAD'>, 'LOCK': <TokenType.LOCK: 'LOCK'>, 'MERGE': <TokenType.MERGE: 'MERGE'>, 'NATURAL': <TokenType.NATURAL: 'NATURAL'>, 'NEXT': <TokenType.NEXT: 'NEXT'>, 'NOT': <TokenType.NOT: 'NOT'>, 'NOTNULL': <TokenType.NOTNULL: 'NOTNULL'>, 'NULL': <TokenType.NULL: 'NULL'>, 'OBJECT': <TokenType.OBJECT: 'OBJECT'>, 'OFFSET': <TokenType.OFFSET: 'OFFSET'>, 'ON': <TokenType.ON: 'ON'>, 'OR': <TokenType.OR: 'OR'>, 'XOR': <TokenType.XOR: 'XOR'>, 'ORDER BY': <TokenType.ORDER_BY: 'ORDER_BY'>, 'ORDINALITY': <TokenType.ORDINALITY: 'ORDINALITY'>, 'OUTER': <TokenType.OUTER: 'OUTER'>, 'OVER': <TokenType.OVER: 'OVER'>, 'OVERLAPS': <TokenType.OVERLAPS: 'OVERLAPS'>, 'OVERWRITE': <TokenType.OVERWRITE: 'OVERWRITE'>, 'PARTITION': <TokenType.PARTITION: 'PARTITION'>, 'PARTITION BY': <TokenType.PARTITION_BY: 'PARTITION_BY'>, 'PARTITIONED BY': <TokenType.PARTITION_BY: 'PARTITION_BY'>, 'PARTITIONED_BY': <TokenType.PARTITION_BY: 'PARTITION_BY'>, 'PERCENT': <TokenType.PERCENT: 'PERCENT'>, 'PIVOT': <TokenType.PIVOT: 'PIVOT'>, 'PRAGMA': <TokenType.PRAGMA: 'PRAGMA'>, 'PRIMARY KEY': <TokenType.PRIMARY_KEY: 'PRIMARY_KEY'>, 'PROCEDURE': <TokenType.PROCEDURE: 'PROCEDURE'>, 'RANGE': <TokenType.RANGE: 'RANGE'>, 'RECURSIVE': <TokenType.RECURSIVE: 'RECURSIVE'>, 'REGEXP': <TokenType.RLIKE: 'RLIKE'>, 'REPLACE': <TokenType.REPLACE: 'REPLACE'>, 'RETURNING': <TokenType.RETURNING: 'RETURNING'>, 'REFERENCES': <TokenType.REFERENCES: 'REFERENCES'>, 'RIGHT': <TokenType.RIGHT: 'RIGHT'>, 'RLIKE': <TokenType.RLIKE: 'RLIKE'>, 'ROLLBACK': <TokenType.ROLLBACK: 'ROLLBACK'>, 'ROLLUP': <TokenType.ROLLUP: 'ROLLUP'>, 'ROW': <TokenType.STRUCT: 'STRUCT'>, 'ROWS': <TokenType.ROWS: 'ROWS'>, 'SCHEMA': <TokenType.SCHEMA: 'SCHEMA'>, 'SELECT': <TokenType.SELECT: 'SELECT'>, 'SEMI': <TokenType.SEMI: 'SEMI'>, 'SET': <TokenType.SET: 'SET'>, 'SETTINGS': <TokenType.SETTINGS: 'SETTINGS'>, 'SHOW': <TokenType.SHOW: 'SHOW'>, 'SIMILAR TO': <TokenType.SIMILAR_TO: 'SIMILAR_TO'>, 'SOME': <TokenType.SOME: 'SOME'>, 'SORT BY': <TokenType.SORT_BY: 'SORT_BY'>, 'START WITH': <TokenType.START_WITH: 'START_WITH'>, 'STRAIGHT_JOIN': <TokenType.STRAIGHT_JOIN: 'STRAIGHT_JOIN'>, 'TABLE': <TokenType.TABLE: 'TABLE'>, 'TABLESAMPLE': <TokenType.TABLE_SAMPLE: 'TABLE_SAMPLE'>, 'TEMP': <TokenType.TEMPORARY: 'TEMPORARY'>, 'TEMPORARY': <TokenType.TEMPORARY: 'TEMPORARY'>, 'THEN': <TokenType.THEN: 'THEN'>, 'TRUE': <TokenType.TRUE: 'TRUE'>, 'TRUNCATE': <TokenType.TRUNCATE: 'TRUNCATE'>, 'UNION': <TokenType.UNION: 'UNION'>, 'UNKNOWN': <TokenType.UNKNOWN: 'UNKNOWN'>, 'UNNEST': <TokenType.UNNEST: 'UNNEST'>, 'UNPIVOT': <TokenType.UNPIVOT: 'UNPIVOT'>, 'UPDATE': <TokenType.UPDATE: 'UPDATE'>, 'USE': <TokenType.USE: 'USE'>, 'USING': <TokenType.USING: 'USING'>, 'UUID': <TokenType.UUID: 'UUID'>, 'VALUES': <TokenType.VALUES: 'VALUES'>, 'VIEW': <TokenType.VIEW: 'VIEW'>, 'VOLATILE': <TokenType.VOLATILE: 'VOLATILE'>, 'WHEN': <TokenType.WHEN: 'WHEN'>, 'WHERE': <TokenType.WHERE: 'WHERE'>, 'WINDOW': <TokenType.WINDOW: 'WINDOW'>, 'WITH': <TokenType.WITH: 'WITH'>, 'APPLY': <TokenType.APPLY: 'APPLY'>, 'ARRAY': <TokenType.ARRAY: 'ARRAY'>, 'BIT': <TokenType.BIT: 'BIT'>, 'BOOL': <TokenType.BOOLEAN: 'BOOLEAN'>, 'BOOLEAN': <TokenType.BOOLEAN: 'BOOLEAN'>, 'BYTE': <TokenType.TINYINT: 'TINYINT'>, 'MEDIUMINT': <TokenType.MEDIUMINT: 'MEDIUMINT'>, 'INT1': <TokenType.TINYINT: 'TINYINT'>, 'TINYINT': <TokenType.TINYINT: 'TINYINT'>, 'INT16': <TokenType.SMALLINT: 'SMALLINT'>, 'SHORT': <TokenType.SMALLINT: 'SMALLINT'>, 'SMALLINT': <TokenType.SMALLINT: 'SMALLINT'>, 'INT128': <TokenType.INT128: 'INT128'>, 'HUGEINT': <TokenType.INT128: 'INT128'>, 'INT2': <TokenType.SMALLINT: 'SMALLINT'>, 'INTEGER': <TokenType.INT: 'INT'>, 'INT': <TokenType.INT: 'INT'>, 'INT4': <TokenType.INT: 'INT'>, 'INT32': <TokenType.INT: 'INT'>, 'INT64': <TokenType.BIGINT: 'BIGINT'>, 'LONG': <TokenType.BIGINT: 'BIGINT'>, 'BIGINT': <TokenType.BIGINT: 'BIGINT'>, 'INT8': <TokenType.TINYINT: 'TINYINT'>, 'UINT': <TokenType.UINT: 'UINT'>, 'DEC': <TokenType.DECIMAL: 'DECIMAL'>, 'DECIMAL': <TokenType.DECIMAL: 'DECIMAL'>, 'BIGDECIMAL': <TokenType.BIGDECIMAL: 'BIGDECIMAL'>, 'BIGNUMERIC': <TokenType.BIGDECIMAL: 'BIGDECIMAL'>, 'LIST': <TokenType.LIST: 'LIST'>, 'MAP': <TokenType.MAP: 'MAP'>, 'NULLABLE': <TokenType.NULLABLE: 'NULLABLE'>, 'NUMBER': <TokenType.DECIMAL: 'DECIMAL'>, 'NUMERIC': <TokenType.DECIMAL: 'DECIMAL'>, 'FIXED': <TokenType.DECIMAL: 'DECIMAL'>, 'REAL': <TokenType.FLOAT: 'FLOAT'>, 'FLOAT': <TokenType.FLOAT: 'FLOAT'>, 'FLOAT4': <TokenType.FLOAT: 'FLOAT'>, 'FLOAT8': <TokenType.DOUBLE: 'DOUBLE'>, 'DOUBLE': <TokenType.DOUBLE: 'DOUBLE'>, 'DOUBLE PRECISION': <TokenType.DOUBLE: 'DOUBLE'>, 'JSON': <TokenType.JSON: 'JSON'>, 'JSONB': <TokenType.JSONB: 'JSONB'>, 'CHAR': <TokenType.CHAR: 'CHAR'>, 'CHARACTER': <TokenType.CHAR: 'CHAR'>, 'NCHAR': <TokenType.NCHAR: 'NCHAR'>, 'VARCHAR': <TokenType.VARCHAR: 'VARCHAR'>, 'VARCHAR2': <TokenType.VARCHAR: 'VARCHAR'>, 'NVARCHAR': <TokenType.NVARCHAR: 'NVARCHAR'>, 'NVARCHAR2': <TokenType.NVARCHAR: 'NVARCHAR'>, 'BPCHAR': <TokenType.BPCHAR: 'BPCHAR'>, 'STR': <TokenType.TEXT: 'TEXT'>, 'STRING': <TokenType.TEXT: 'TEXT'>, 'TEXT': <TokenType.TEXT: 'TEXT'>, 'LONGTEXT': <TokenType.LONGTEXT: 'LONGTEXT'>, 'MEDIUMTEXT': <TokenType.MEDIUMTEXT: 'MEDIUMTEXT'>, 'TINYTEXT': <TokenType.TINYTEXT: 'TINYTEXT'>, 'CLOB': <TokenType.TEXT: 'TEXT'>, 'LONGVARCHAR': <TokenType.TEXT: 'TEXT'>, 'BINARY': <TokenType.BINARY: 'BINARY'>, 'BLOB': <TokenType.VARBINARY: 'VARBINARY'>, 'LONGBLOB': <TokenType.LONGBLOB: 'LONGBLOB'>, 'MEDIUMBLOB': <TokenType.MEDIUMBLOB: 'MEDIUMBLOB'>, 'TINYBLOB': <TokenType.TINYBLOB: 'TINYBLOB'>, 'BYTEA': <TokenType.VARBINARY: 'VARBINARY'>, 'VARBINARY': <TokenType.VARBINARY: 'VARBINARY'>, 'TIME': <TokenType.TIME: 'TIME'>, 'TIMETZ': <TokenType.TIMETZ: 'TIMETZ'>, 'TIMESTAMP': <TokenType.TIMESTAMP: 'TIMESTAMP'>, 'TIMESTAMPTZ': <TokenType.TIMESTAMPTZ: 'TIMESTAMPTZ'>, 'TIMESTAMPLTZ': <TokenType.TIMESTAMPLTZ: 'TIMESTAMPLTZ'>, 'TIMESTAMP_LTZ': <TokenType.TIMESTAMPLTZ: 'TIMESTAMPLTZ'>, 'TIMESTAMPNTZ': <TokenType.TIMESTAMPNTZ: 'TIMESTAMPNTZ'>, 'TIMESTAMP_NTZ': <TokenType.TIMESTAMPNTZ: 'TIMESTAMPNTZ'>, 'DATE': <TokenType.DATE: 'DATE'>, 'DATETIME': <TokenType.DATETIME: 'DATETIME'>, 'INT4RANGE': <TokenType.INT4RANGE: 'INT4RANGE'>, 'INT4MULTIRANGE': <TokenType.INT4MULTIRANGE: 'INT4MULTIRANGE'>, 'INT8RANGE': <TokenType.INT8RANGE: 'INT8RANGE'>, 'INT8MULTIRANGE': <TokenType.INT8MULTIRANGE: 'INT8MULTIRANGE'>, 'NUMRANGE': <TokenType.NUMRANGE: 'NUMRANGE'>, 'NUMMULTIRANGE': <TokenType.NUMMULTIRANGE: 'NUMMULTIRANGE'>, 'TSRANGE': <TokenType.TSRANGE: 'TSRANGE'>, 'TSMULTIRANGE': <TokenType.TSMULTIRANGE: 'TSMULTIRANGE'>, 'TSTZRANGE': <TokenType.TSTZRANGE: 'TSTZRANGE'>, 'TSTZMULTIRANGE': <TokenType.TSTZMULTIRANGE: 'TSTZMULTIRANGE'>, 'DATERANGE': <TokenType.DATERANGE: 'DATERANGE'>, 'DATEMULTIRANGE': <TokenType.DATEMULTIRANGE: 'DATEMULTIRANGE'>, 'UNIQUE': <TokenType.UNIQUE: 'UNIQUE'>, 'VECTOR': <TokenType.VECTOR: 'VECTOR'>, 'STRUCT': <TokenType.STRUCT: 'STRUCT'>, 'SEQUENCE': <TokenType.SEQUENCE: 'SEQUENCE'>, 'VARIANT': <TokenType.VARIANT: 'VARIANT'>, 'ALTER': <TokenType.ALTER: 'ALTER'>, 'ANALYZE': <TokenType.COMMAND: 'COMMAND'>, 'CALL': <TokenType.COMMAND: 'COMMAND'>, 'COMMENT': <TokenType.COMMENT: 'COMMENT'>, 'EXPLAIN': <TokenType.COMMAND: 'COMMAND'>, 'GRANT': <TokenType.COMMAND: 'COMMAND'>, 'OPTIMIZE': <TokenType.COMMAND: 'COMMAND'>, 'PREPARE': <TokenType.COMMAND: 'COMMAND'>, 'VACUUM': <TokenType.COMMAND: 'COMMAND'>, 'USER-DEFINED': <TokenType.USERDEFINED: 'USERDEFINED'>, 'FOR VERSION': <TokenType.VERSION_SNAPSHOT: 'VERSION_SNAPSHOT'>, 'FOR TIMESTAMP': <TokenType.TIMESTAMP_SNAPSHOT: 'TIMESTAMP_SNAPSHOT'>, 'START': <TokenType.BEGIN: 'BEGIN'>, 'MATCH_RECOGNIZE': <TokenType.MATCH_RECOGNIZE: 'MATCH_RECOGNIZE'>, 'IPADDRESS': <TokenType.IPADDRESS: 'IPADDRESS'>, 'IPPREFIX': <TokenType.IPPREFIX: 'IPPREFIX'>, 'TDIGEST': <TokenType.TDIGEST: 'TDIGEST'>, 'HYPERLOGLOG': <TokenType.HLLSKETCH: 'HLLSKETCH'>}

##### Inherited Members

sqlglot.tokens.Tokenizer
  Tokenizer
  SINGLE_TOKENS
  BIT_STRINGS
  BYTE_STRINGS
  HEX_STRINGS
  RAW_STRINGS
  HEREDOC_STRINGS
  IDENTIFIERS
  IDENTIFIER_ESCAPES
  QUOTES
  STRING_ESCAPES
  VAR_SINGLE_TOKENS
  HEREDOC_TAG_IS_IDENTIFIER
  HEREDOC_STRING_ALTERNATIVE
  STRING_ESCAPES_ALLOWED_IN_RAW_STRINGS
  WHITE_SPACE
  COMMANDS
  COMMAND_PREFIX_TOKENS
  NUMERIC_LITERALS
  COMMENTS
  dialect
  reset
  tokenize
  tokenize_rs
  size
  sql
  tokens

class Presto.Parser(sqlglot.parser.Parser): View Source

    296    class Parser(parser.Parser):
    297        VALUES_FOLLOWED_BY_PAREN = False
    298
    299        FUNCTIONS = {
    300            **parser.Parser.FUNCTIONS,
    301            "ARBITRARY": exp.AnyValue.from_arg_list,
    302            "APPROX_DISTINCT": exp.ApproxDistinct.from_arg_list,
    303            "APPROX_PERCENTILE": _build_approx_percentile,
    304            "BITWISE_AND": binary_from_function(exp.BitwiseAnd),
    305            "BITWISE_NOT": lambda args: exp.BitwiseNot(this=seq_get(args, 0)),
    306            "BITWISE_OR": binary_from_function(exp.BitwiseOr),
    307            "BITWISE_XOR": binary_from_function(exp.BitwiseXor),
    308            "CARDINALITY": exp.ArraySize.from_arg_list,
    309            "CONTAINS": exp.ArrayContains.from_arg_list,
    310            "DATE_ADD": lambda args: exp.DateAdd(
    311                this=seq_get(args, 2), expression=seq_get(args, 1), unit=seq_get(args, 0)
    312            ),
    313            "DATE_DIFF": lambda args: exp.DateDiff(
    314                this=seq_get(args, 2), expression=seq_get(args, 1), unit=seq_get(args, 0)
    315            ),
    316            "DATE_FORMAT": build_formatted_time(exp.TimeToStr, "presto"),
    317            "DATE_PARSE": build_formatted_time(exp.StrToTime, "presto"),
    318            "DATE_TRUNC": date_trunc_to_time,
    319            "ELEMENT_AT": lambda args: exp.Bracket(
    320                this=seq_get(args, 0), expressions=[seq_get(args, 1)], offset=1, safe=True
    321            ),
    322            "FROM_HEX": exp.Unhex.from_arg_list,
    323            "FROM_UNIXTIME": _build_from_unixtime,
    324            "FROM_UTF8": lambda args: exp.Decode(
    325                this=seq_get(args, 0), replace=seq_get(args, 1), charset=exp.Literal.string("utf-8")
    326            ),
    327            "NOW": exp.CurrentTimestamp.from_arg_list,
    328            "REGEXP_EXTRACT": lambda args: exp.RegexpExtract(
    329                this=seq_get(args, 0), expression=seq_get(args, 1), group=seq_get(args, 2)
    330            ),
    331            "REGEXP_REPLACE": lambda args: exp.RegexpReplace(
    332                this=seq_get(args, 0),
    333                expression=seq_get(args, 1),
    334                replacement=seq_get(args, 2) or exp.Literal.string(""),
    335            ),
    336            "ROW": exp.Struct.from_arg_list,
    337            "SEQUENCE": exp.GenerateSeries.from_arg_list,
    338            "SET_AGG": exp.ArrayUniqueAgg.from_arg_list,
    339            "SPLIT_TO_MAP": exp.StrToMap.from_arg_list,
    340            "STRPOS": lambda args: exp.StrPosition(
    341                this=seq_get(args, 0), substr=seq_get(args, 1), instance=seq_get(args, 2)
    342            ),
    343            "TO_CHAR": _build_to_char,
    344            "TO_UNIXTIME": exp.TimeToUnix.from_arg_list,
    345            "TO_UTF8": lambda args: exp.Encode(
    346                this=seq_get(args, 0), charset=exp.Literal.string("utf-8")
    347            ),
    348            "MD5": exp.MD5Digest.from_arg_list,
    349            "SHA256": lambda args: exp.SHA2(this=seq_get(args, 0), length=exp.Literal.number(256)),
    350            "SHA512": lambda args: exp.SHA2(this=seq_get(args, 0), length=exp.Literal.number(512)),
    351        }
    352
    353        FUNCTION_PARSERS = parser.Parser.FUNCTION_PARSERS.copy()
    354        FUNCTION_PARSERS.pop("TRIM")

Parser consumes a list of tokens produced by the Tokenizer and produces a parsed syntax tree.

###### Arguments:

- error_level: The desired error level. Default: ErrorLevel.IMMEDIATE
- error_message_context: The amount of context to capture from a query string when displaying the error message (in number of characters). Default: 100
- max_errors: Maximum number of error messages to include in a raised ParseError. This is only relevant if error_level is ErrorLevel.RAISE. Default: 3

VALUES_FOLLOWED_BY_PAREN = False
FUNCTIONS = {'ABS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Abs'>>, 'ADD_MONTHS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.AddMonths'>>, 'ANONYMOUS_AGG_FUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.AnonymousAggFunc'>>, 'ANY_VALUE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.AnyValue'>>, 'APPROX_DISTINCT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ApproxDistinct'>>, 'APPROX_COUNT_DISTINCT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ApproxDistinct'>>, 'APPROX_QUANTILE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ApproxQuantile'>>, 'APPROX_TOP_K': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ApproxTopK'>>, 'ARG_MAX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArgMax'>>, 'ARGMAX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArgMax'>>, 'MAX_BY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArgMax'>>, 'ARG_MIN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArgMin'>>, 'ARGMIN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArgMin'>>, 'MIN_BY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArgMin'>>, 'ARRAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Array'>>, 'ARRAY_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayAgg'>>, 'ARRAY_ALL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayAll'>>, 'ARRAY_ANY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayAny'>>, 'ARRAY_CONCAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayConcat'>>, 'ARRAY_CAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayConcat'>>, 'ARRAY_CONSTRUCT_COMPACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayConstructCompact'>>, 'ARRAY_CONTAINS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayContains'>>, 'ARRAY_HAS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayContains'>>, 'ARRAY_CONTAINS_ALL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayContainsAll'>>, 'ARRAY_HAS_ALL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayContainsAll'>>, 'FILTER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayFilter'>>, 'ARRAY_FILTER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayFilter'>>, 'ARRAY_OVERLAPS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayOverlaps'>>, 'ARRAY_SIZE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArraySize'>>, 'ARRAY_LENGTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArraySize'>>, 'ARRAY_SORT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArraySort'>>, 'ARRAY_SUM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArraySum'>>, 'ARRAY_TO_STRING': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayToString'>>, 'ARRAY_JOIN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayToString'>>, 'ARRAY_UNION_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayUnionAgg'>>, 'ARRAY_UNIQUE_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayUniqueAgg'>>, 'AVG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Avg'>>, 'CASE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Case'>>, 'CAST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Cast'>>, 'CAST_TO_STR_TYPE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CastToStrType'>>, 'CBRT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Cbrt'>>, 'CEIL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Ceil'>>, 'CEILING': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Ceil'>>, 'CHR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Chr'>>, 'CHAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Chr'>>, 'COALESCE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Coalesce'>>, 'IFNULL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Coalesce'>>, 'NVL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Coalesce'>>, 'COLLATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Collate'>>, 'COMBINED_AGG_FUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CombinedAggFunc'>>, 'COMBINED_PARAMETERIZED_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CombinedParameterizedAgg'>>, 'CONCAT': <function Parser.<lambda>>, 'CONCAT_WS': <function Parser.<lambda>>, 'CONNECT_BY_ROOT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ConnectByRoot'>>, 'CONVERT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Convert'>>, 'CORR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Corr'>>, 'COUNT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Count'>>, 'COUNT_IF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CountIf'>>, 'COUNTIF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CountIf'>>, 'COVAR_POP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CovarPop'>>, 'COVAR_SAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CovarSamp'>>, 'CURRENT_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentDate'>>, 'CURRENT_DATETIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentDatetime'>>, 'CURRENT_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentTime'>>, 'CURRENT_TIMESTAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentTimestamp'>>, 'CURRENT_USER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentUser'>>, 'DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Date'>>, 'DATE_ADD': <function Presto.Parser.<lambda>>, 'DATEDIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateDiff'>>, 'DATE_DIFF': <function Presto.Parser.<lambda>>, 'DATE_FROM_PARTS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateFromParts'>>, 'DATEFROMPARTS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateFromParts'>>, 'DATE_STR_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateStrToDate'>>, 'DATE_SUB': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateSub'>>, 'DATE_TO_DATE_STR': <function Parser.<lambda>>, 'DATE_TO_DI': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateToDi'>>, 'DATE_TRUNC': <function date_trunc_to_time>, 'DATETIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Datetime'>>, 'DATETIME_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DatetimeAdd'>>, 'DATETIME_DIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DatetimeDiff'>>, 'DATETIME_SUB': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DatetimeSub'>>, 'DATETIME_TRUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DatetimeTrunc'>>, 'DAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Day'>>, 'DAY_OF_MONTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfMonth'>>, 'DAYOFMONTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfMonth'>>, 'DAY_OF_WEEK': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfWeek'>>, 'DAYOFWEEK': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfWeek'>>, 'DAY_OF_YEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfYear'>>, 'DAYOFYEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfYear'>>, 'DECODE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Decode'>>, 'DI_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DiToDate'>>, 'ENCODE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Encode'>>, 'EXP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Exp'>>, 'EXPLODE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Explode'>>, 'EXPLODE_OUTER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ExplodeOuter'>>, 'EXTRACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Extract'>>, 'FIRST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.First'>>, 'FIRST_VALUE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.FirstValue'>>, 'FLATTEN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Flatten'>>, 'FLOOR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Floor'>>, 'FROM_BASE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.FromBase'>>, 'FROM_BASE64': <bound method Func.from_arg_list of <class 'sqlglot.expressions.FromBase64'>>, 'GAP_FILL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.GapFill'>>, 'GENERATE_DATE_ARRAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.GenerateDateArray'>>, 'GENERATE_SERIES': <bound method Func.from_arg_list of <class 'sqlglot.expressions.GenerateSeries'>>, 'GREATEST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Greatest'>>, 'GROUP_CONCAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.GroupConcat'>>, 'HEX': <function build_hex>, 'HLL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Hll'>>, 'IF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.If'>>, 'IIF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.If'>>, 'INITCAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Initcap'>>, 'IS_INF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.IsInf'>>, 'ISINF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.IsInf'>>, 'IS_NAN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.IsNan'>>, 'ISNAN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.IsNan'>>, 'J_S_O_N_ARRAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONArray'>>, 'J_S_O_N_ARRAY_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONArrayAgg'>>, 'JSON_ARRAY_CONTAINS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONArrayContains'>>, 'JSONB_CONTAINS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONBContains'>>, 'JSONB_EXTRACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONBExtract'>>, 'JSONB_EXTRACT_SCALAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONBExtractScalar'>>, 'JSON_EXTRACT': <function build_extract_json_with_path.<locals>._builder>, 'JSON_EXTRACT_SCALAR': <function build_extract_json_with_path.<locals>._builder>, 'JSON_FORMAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONFormat'>>, 'J_S_O_N_OBJECT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONObject'>>, 'J_S_O_N_OBJECT_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONObjectAgg'>>, 'J_S_O_N_TABLE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONTable'>>, 'LAG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Lag'>>, 'LAST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Last'>>, 'LAST_DAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LastDay'>>, 'LAST_DAY_OF_MONTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LastDay'>>, 'LAST_VALUE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LastValue'>>, 'LEAD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Lead'>>, 'LEAST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Least'>>, 'LEFT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Left'>>, 'LENGTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Length'>>, 'LEN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Length'>>, 'LEVENSHTEIN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Levenshtein'>>, 'LIST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.List'>>, 'LN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Ln'>>, 'LOG': <function build_logarithm>, 'LOGICAL_AND': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalAnd'>>, 'BOOL_AND': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalAnd'>>, 'BOOLAND_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalAnd'>>, 'LOGICAL_OR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalOr'>>, 'BOOL_OR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalOr'>>, 'BOOLOR_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalOr'>>, 'LOWER': <function build_lower>, 'LCASE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Lower'>>, 'LOWER_HEX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LowerHex'>>, 'MD5': <bound method Func.from_arg_list of <class 'sqlglot.expressions.MD5Digest'>>, 'MD5_DIGEST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.MD5Digest'>>, 'MAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Map'>>, 'MAP_FROM_ENTRIES': <bound method Func.from_arg_list of <class 'sqlglot.expressions.MapFromEntries'>>, 'MATCH_AGAINST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.MatchAgainst'>>, 'MAX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Max'>>, 'MIN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Min'>>, 'MONTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Month'>>, 'MONTHS_BETWEEN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.MonthsBetween'>>, 'NEXT_VALUE_FOR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.NextValueFor'>>, 'NTH_VALUE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.NthValue'>>, 'NULLIF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Nullif'>>, 'NUMBER_TO_STR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.NumberToStr'>>, 'NVL2': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Nvl2'>>, 'OBJECT_INSERT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ObjectInsert'>>, 'OPEN_J_S_O_N': <bound method Func.from_arg_list of <class 'sqlglot.expressions.OpenJSON'>>, 'PAD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Pad'>>, 'PARAMETERIZED_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ParameterizedAgg'>>, 'PARSE_JSON': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ParseJSON'>>, 'JSON_PARSE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ParseJSON'>>, 'PERCENTILE_CONT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.PercentileCont'>>, 'PERCENTILE_DISC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.PercentileDisc'>>, 'POSEXPLODE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Posexplode'>>, 'POSEXPLODE_OUTER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.PosexplodeOuter'>>, 'POWER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Pow'>>, 'POW': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Pow'>>, 'PREDICT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Predict'>>, 'QUANTILE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Quantile'>>, 'QUARTER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Quarter'>>, 'RAND': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Rand'>>, 'RANDOM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Rand'>>, 'RANDN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Randn'>>, 'RANGE_N': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RangeN'>>, 'READ_CSV': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ReadCSV'>>, 'REDUCE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Reduce'>>, 'REGEXP_EXTRACT': <function Presto.Parser.<lambda>>, 'REGEXP_I_LIKE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RegexpILike'>>, 'REGEXP_LIKE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RegexpLike'>>, 'REGEXP_REPLACE': <function Presto.Parser.<lambda>>, 'REGEXP_SPLIT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RegexpSplit'>>, 'REPEAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Repeat'>>, 'RIGHT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Right'>>, 'ROUND': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Round'>>, 'ROW_NUMBER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RowNumber'>>, 'SHA': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SHA'>>, 'SHA1': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SHA'>>, 'SHA2': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SHA2'>>, 'SAFE_DIVIDE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SafeDivide'>>, 'SIGN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Sign'>>, 'SIGNUM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Sign'>>, 'SORT_ARRAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SortArray'>>, 'SPLIT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Split'>>, 'SQRT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Sqrt'>>, 'STANDARD_HASH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StandardHash'>>, 'STAR_MAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StarMap'>>, 'STARTS_WITH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StartsWith'>>, 'STARTSWITH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StartsWith'>>, 'STDDEV': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Stddev'>>, 'STDEV': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Stddev'>>, 'STDDEV_POP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StddevPop'>>, 'STDDEV_SAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StddevSamp'>>, 'STR_POSITION': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrPosition'>>, 'STR_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrToDate'>>, 'STR_TO_MAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrToMap'>>, 'STR_TO_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrToTime'>>, 'STR_TO_UNIX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrToUnix'>>, 'STRING_TO_ARRAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StringToArray'>>, 'SPLIT_BY_STRING': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StringToArray'>>, 'STRUCT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Struct'>>, 'STRUCT_EXTRACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StructExtract'>>, 'STUFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Stuff'>>, 'INSERT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Stuff'>>, 'SUBSTRING': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Substring'>>, 'SUM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Sum'>>, 'TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Time'>>, 'TIME_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeAdd'>>, 'TIME_DIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeDiff'>>, 'TIME_FROM_PARTS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeFromParts'>>, 'TIMEFROMPARTS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeFromParts'>>, 'TIME_STR_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeStrToDate'>>, 'TIME_STR_TO_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeStrToTime'>>, 'TIME_STR_TO_UNIX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeStrToUnix'>>, 'TIME_SUB': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeSub'>>, 'TIME_TO_STR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeToStr'>>, 'TIME_TO_TIME_STR': <function Parser.<lambda>>, 'TIME_TO_UNIX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeToUnix'>>, 'TIME_TRUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeTrunc'>>, 'TIMESTAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Timestamp'>>, 'TIMESTAMP_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampAdd'>>, 'TIMESTAMPDIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampDiff'>>, 'TIMESTAMP_DIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampDiff'>>, 'TIMESTAMP_FROM_PARTS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampFromParts'>>, 'TIMESTAMPFROMPARTS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampFromParts'>>, 'TIMESTAMP_SUB': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampSub'>>, 'TIMESTAMP_TRUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampTrunc'>>, 'TO_ARRAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ToArray'>>, 'TO_BASE64': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ToBase64'>>, 'TO_CHAR': <function _build_to_char>, 'TO_DAYS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ToDays'>>, 'TO_MAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ToMap'>>, 'TO_NUMBER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ToNumber'>>, 'TRANSFORM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Transform'>>, 'TRIM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Trim'>>, 'TRY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Try'>>, 'TRY_CAST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TryCast'>>, 'TS_OR_DI_TO_DI': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDiToDi'>>, 'TS_OR_DS_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDsAdd'>>, 'TS_OR_DS_DIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDsDiff'>>, 'TS_OR_DS_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDsToDate'>>, 'TS_OR_DS_TO_DATE_STR': <function Parser.<lambda>>, 'TS_OR_DS_TO_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDsToTime'>>, 'TS_OR_DS_TO_TIMESTAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDsToTimestamp'>>, 'UNHEX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Unhex'>>, 'UNIX_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.UnixDate'>>, 'UNIX_TO_STR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.UnixToStr'>>, 'UNIX_TO_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.UnixToTime'>>, 'UNIX_TO_TIME_STR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.UnixToTimeStr'>>, 'UNNEST': <function Parser.<lambda>>, 'UPPER': <function build_upper>, 'UCASE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Upper'>>, 'VAR_MAP': <function build_var_map>, 'VARIANCE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Variance'>>, 'VARIANCE_SAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Variance'>>, 'VAR_SAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Variance'>>, 'VARIANCE_POP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.VariancePop'>>, 'VAR_POP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.VariancePop'>>, 'WEEK': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Week'>>, 'WEEK_OF_YEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.WeekOfYear'>>, 'WEEKOFYEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.WeekOfYear'>>, 'WHEN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.When'>>, 'X_M_L_TABLE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.XMLTable'>>, 'XOR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Xor'>>, 'YEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Year'>>, 'GLOB': <function Parser.<lambda>>, 'JSON_EXTRACT_PATH_TEXT': <function build_extract_json_with_path.<locals>._builder>, 'LIKE': <function build_like>, 'LOG2': <function Parser.<lambda>>, 'LOG10': <function Parser.<lambda>>, 'LPAD': <function Parser.<lambda>>, 'LEFTPAD': <function Parser.<lambda>>, 'MOD': <function build_mod>, 'RPAD': <function Parser.<lambda>>, 'RIGHTPAD': <function Parser.<lambda>>, 'SCOPE_RESOLUTION': <function Parser.<lambda>>, 'TO_HEX': <function build_hex>, 'ARBITRARY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.AnyValue'>>, 'APPROX_PERCENTILE': <function _build_approx_percentile>, 'BITWISE_AND': <function binary_from_function.<locals>.<lambda>>, 'BITWISE_NOT': <function Presto.Parser.<lambda>>, 'BITWISE_OR': <function binary_from_function.<locals>.<lambda>>, 'BITWISE_XOR': <function binary_from_function.<locals>.<lambda>>, 'CARDINALITY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArraySize'>>, 'CONTAINS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayContains'>>, 'DATE_FORMAT': <function build_formatted_time.<locals>._builder>, 'DATE_PARSE': <function build_formatted_time.<locals>._builder>, 'ELEMENT_AT': <function Presto.Parser.<lambda>>, 'FROM_HEX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Unhex'>>, 'FROM_UNIXTIME': <function _build_from_unixtime>, 'FROM_UTF8': <function Presto.Parser.<lambda>>, 'NOW': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentTimestamp'>>, 'ROW': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Struct'>>, 'SEQUENCE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.GenerateSeries'>>, 'SET_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayUniqueAgg'>>, 'SPLIT_TO_MAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrToMap'>>, 'STRPOS': <function Presto.Parser.<lambda>>, 'TO_UNIXTIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeToUnix'>>, 'TO_UTF8': <function Presto.Parser.<lambda>>, 'SHA256': <function Presto.Parser.<lambda>>, 'SHA512': <function Presto.Parser.<lambda>>}
FUNCTION_PARSERS = {'CAST': <function Parser.<lambda>>, 'CONVERT': <function Parser.<lambda>>, 'DECODE': <function Parser.<lambda>>, 'EXTRACT': <function Parser.<lambda>>, 'GAP_FILL': <function Parser.<lambda>>, 'JSON_OBJECT': <function Parser.<lambda>>, 'JSON_OBJECTAGG': <function Parser.<lambda>>, 'JSON_TABLE': <function Parser.<lambda>>, 'MATCH': <function Parser.<lambda>>, 'OPENJSON': <function Parser.<lambda>>, 'POSITION': <function Parser.<lambda>>, 'PREDICT': <function Parser.<lambda>>, 'SAFE_CAST': <function Parser.<lambda>>, 'STRING_AGG': <function Parser.<lambda>>, 'SUBSTRING': <function Parser.<lambda>>, 'TRY_CAST': <function Parser.<lambda>>, 'TRY_CONVERT': <function Parser.<lambda>>}
ID_VAR_TOKENS = {<TokenType.DATE32: 'DATE32'>, <TokenType.MAP: 'MAP'>, <TokenType.WINDOW: 'WINDOW'>, <TokenType.UMEDIUMINT: 'UMEDIUMINT'>, <TokenType.COPY: 'COPY'>, <TokenType.UBIGINT: 'UBIGINT'>, <TokenType.PSEUDO_TYPE: 'PSEUDO_TYPE'>, <TokenType.UNKNOWN: 'UNKNOWN'>, <TokenType.DICTIONARY: 'DICTIONARY'>, <TokenType.UNIQUEIDENTIFIER: 'UNIQUEIDENTIFIER'>, <TokenType.TDIGEST: 'TDIGEST'>, <TokenType.IS: 'IS'>, <TokenType.XML: 'XML'>, <TokenType.VIEW: 'VIEW'>, <TokenType.TIMESTAMPTZ: 'TIMESTAMPTZ'>, <TokenType.ROWVERSION: 'ROWVERSION'>, <TokenType.REFERENCES: 'REFERENCES'>, <TokenType.TOP: 'TOP'>, <TokenType.UUID: 'UUID'>, <TokenType.DIV: 'DIV'>, <TokenType.IPADDRESS: 'IPADDRESS'>, <TokenType.SERIAL: 'SERIAL'>, <TokenType.VARCHAR: 'VARCHAR'>, <TokenType.INT4RANGE: 'INT4RANGE'>, <TokenType.BIT: 'BIT'>, <TokenType.ROW: 'ROW'>, <TokenType.DATETIME: 'DATETIME'>, <TokenType.BPCHAR: 'BPCHAR'>, <TokenType.SEMI: 'SEMI'>, <TokenType.OPERATOR: 'OPERATOR'>, <TokenType.SOME: 'SOME'>, <TokenType.NUMMULTIRANGE: 'NUMMULTIRANGE'>, <TokenType.LEFT: 'LEFT'>, <TokenType.IMAGE: 'IMAGE'>, <TokenType.CURRENT_DATETIME: 'CURRENT_DATETIME'>, <TokenType.AGGREGATEFUNCTION: 'AGGREGATEFUNCTION'>, <TokenType.TIMESTAMP_NS: 'TIMESTAMP_NS'>, <TokenType.SETTINGS: 'SETTINGS'>, <TokenType.TINYINT: 'TINYINT'>, <TokenType.FOREIGN_KEY: 'FOREIGN_KEY'>, <TokenType.BOOLEAN: 'BOOLEAN'>, <TokenType.BIGDECIMAL: 'BIGDECIMAL'>, <TokenType.DOUBLE: 'DOUBLE'>, <TokenType.MEDIUMBLOB: 'MEDIUMBLOB'>, <TokenType.FALSE: 'FALSE'>, <TokenType.STRAIGHT_JOIN: 'STRAIGHT_JOIN'>, <TokenType.RIGHT: 'RIGHT'>, <TokenType.ORDINALITY: 'ORDINALITY'>, <TokenType.UDECIMAL: 'UDECIMAL'>, <TokenType.NEXT: 'NEXT'>, <TokenType.GEOGRAPHY: 'GEOGRAPHY'>, <TokenType.REPLACE: 'REPLACE'>, <TokenType.USERDEFINED: 'USERDEFINED'>, <TokenType.SHOW: 'SHOW'>, <TokenType.USE: 'USE'>, <TokenType.FIXEDSTRING: 'FIXEDSTRING'>, <TokenType.TIME: 'TIME'>, <TokenType.ESCAPE: 'ESCAPE'>, <TokenType.COMMAND: 'COMMAND'>, <TokenType.TIMESTAMP: 'TIMESTAMP'>, <TokenType.CURRENT_USER: 'CURRENT_USER'>, <TokenType.MERGE: 'MERGE'>, <TokenType.LONGTEXT: 'LONGTEXT'>, <TokenType.INT8RANGE: 'INT8RANGE'>, <TokenType.ARRAY: 'ARRAY'>, <TokenType.NULLABLE: 'NULLABLE'>, <TokenType.ENUM8: 'ENUM8'>, <TokenType.ENUM: 'ENUM'>, <TokenType.ASOF: 'ASOF'>, <TokenType.UTINYINT: 'UTINYINT'>, <TokenType.TIMESTAMP_S: 'TIMESTAMP_S'>, <TokenType.UNNEST: 'UNNEST'>, <TokenType.OBJECT_IDENTIFIER: 'OBJECT_IDENTIFIER'>, <TokenType.UPDATE: 'UPDATE'>, <TokenType.PROCEDURE: 'PROCEDURE'>, <TokenType.ALL: 'ALL'>, <TokenType.APPLY: 'APPLY'>, <TokenType.OVERWRITE: 'OVERWRITE'>, <TokenType.NATURAL: 'NATURAL'>, <TokenType.MODEL: 'MODEL'>, <TokenType.INTERVAL: 'INTERVAL'>, <TokenType.PERCENT: 'PERCENT'>, <TokenType.VECTOR: 'VECTOR'>, <TokenType.OBJECT: 'OBJECT'>, <TokenType.UNIQUE: 'UNIQUE'>, <TokenType.NAME: 'NAME'>, <TokenType.ROLLUP: 'ROLLUP'>, <TokenType.FIRST: 'FIRST'>, <TokenType.TIMESTAMP_MS: 'TIMESTAMP_MS'>, <TokenType.TIMESTAMPLTZ: 'TIMESTAMPLTZ'>, <TokenType.STREAMLIT: 'STREAMLIT'>, <TokenType.LIST: 'LIST'>, <TokenType.RANGE: 'RANGE'>, <TokenType.FORMAT: 'FORMAT'>, <TokenType.YEAR: 'YEAR'>, <TokenType.KEEP: 'KEEP'>, <TokenType.LOAD: 'LOAD'>, <TokenType.PIVOT: 'PIVOT'>, <TokenType.ROWS: 'ROWS'>, <TokenType.SCHEMA: 'SCHEMA'>, <TokenType.EXISTS: 'EXISTS'>, <TokenType.IPV6: 'IPV6'>, <TokenType.UINT: 'UINT'>, <TokenType.INT256: 'INT256'>, <TokenType.TEXT: 'TEXT'>, <TokenType.VARBINARY: 'VARBINARY'>, <TokenType.COLLATE: 'COLLATE'>, <TokenType.BIGINT: 'BIGINT'>, <TokenType.GEOMETRY: 'GEOMETRY'>, <TokenType.VAR: 'VAR'>, <TokenType.CUBE: 'CUBE'>, <TokenType.SET: 'SET'>, <TokenType.SMALLINT: 'SMALLINT'>, <TokenType.IDENTIFIER: 'IDENTIFIER'>, <TokenType.SMALLMONEY: 'SMALLMONEY'>, <TokenType.SMALLSERIAL: 'SMALLSERIAL'>, <TokenType.EXECUTE: 'EXECUTE'>, <TokenType.ASC: 'ASC'>, <TokenType.INET: 'INET'>, <TokenType.NCHAR: 'NCHAR'>, <TokenType.DECIMAL: 'DECIMAL'>, <TokenType.UINT256: 'UINT256'>, <TokenType.WAREHOUSE: 'WAREHOUSE'>, <TokenType.MEDIUMINT: 'MEDIUMINT'>, <TokenType.MEDIUMTEXT: 'MEDIUMTEXT'>, <TokenType.COMMIT: 'COMMIT'>, <TokenType.DELETE: 'DELETE'>, <TokenType.NVARCHAR: 'NVARCHAR'>, <TokenType.INT: 'INT'>, <TokenType.STRUCT: 'STRUCT'>, <TokenType.DEFAULT: 'DEFAULT'>, <TokenType.HLLSKETCH: 'HLLSKETCH'>, <TokenType.INDEX: 'INDEX'>, <TokenType.COMMENT: 'COMMENT'>, <TokenType.BEGIN: 'BEGIN'>, <TokenType.DATETIME64: 'DATETIME64'>, <TokenType.TINYTEXT: 'TINYTEXT'>, <TokenType.FLOAT: 'FLOAT'>, <TokenType.UNPIVOT: 'UNPIVOT'>, <TokenType.TIMETZ: 'TIMETZ'>, <TokenType.TSRANGE: 'TSRANGE'>, <TokenType.MONEY: 'MONEY'>, <TokenType.ENUM16: 'ENUM16'>, <TokenType.HSTORE: 'HSTORE'>, <TokenType.DESCRIBE: 'DESCRIBE'>, <TokenType.TABLE: 'TABLE'>, <TokenType.CURRENT_TIMESTAMP: 'CURRENT_TIMESTAMP'>, <TokenType.REFRESH: 'REFRESH'>, <TokenType.DATE: 'DATE'>, <TokenType.LOWCARDINALITY: 'LOWCARDINALITY'>, <TokenType.END: 'END'>, <TokenType.CACHE: 'CACHE'>, <TokenType.NUMRANGE: 'NUMRANGE'>, <TokenType.ANY: 'ANY'>, <TokenType.ANTI: 'ANTI'>, <TokenType.TINYBLOB: 'TINYBLOB'>, <TokenType.UINT128: 'UINT128'>, <TokenType.JSON: 'JSON'>, <TokenType.BIGSERIAL: 'BIGSERIAL'>, <TokenType.DATEMULTIRANGE: 'DATEMULTIRANGE'>, <TokenType.LONGBLOB: 'LONGBLOB'>, <TokenType.FUNCTION: 'FUNCTION'>, <TokenType.DESC: 'DESC'>, <TokenType.FULL: 'FULL'>, <TokenType.ISNULL: 'ISNULL'>, <TokenType.FINAL: 'FINAL'>, <TokenType.TRUNCATE: 'TRUNCATE'>, <TokenType.OVERLAPS: 'OVERLAPS'>, <TokenType.OFFSET: 'OFFSET'>, <TokenType.TSMULTIRANGE: 'TSMULTIRANGE'>, <TokenType.FILTER: 'FILTER'>, <TokenType.PRAGMA: 'PRAGMA'>, <TokenType.TRUE: 'TRUE'>, <TokenType.INT8MULTIRANGE: 'INT8MULTIRANGE'>, <TokenType.CHAR: 'CHAR'>, <TokenType.SIMPLEAGGREGATEFUNCTION: 'SIMPLEAGGREGATEFUNCTION'>, <TokenType.VARIANT: 'VARIANT'>, <TokenType.SUPER: 'SUPER'>, <TokenType.TEMPORARY: 'TEMPORARY'>, <TokenType.CASE: 'CASE'>, <TokenType.AUTO_INCREMENT: 'AUTO_INCREMENT'>, <TokenType.IPV4: 'IPV4'>, <TokenType.KILL: 'KILL'>, <TokenType.NESTED: 'NESTED'>, <TokenType.BINARY: 'BINARY'>, <TokenType.VOLATILE: 'VOLATILE'>, <TokenType.PARTITION: 'PARTITION'>, <TokenType.DATERANGE: 'DATERANGE'>, <TokenType.TSTZMULTIRANGE: 'TSTZMULTIRANGE'>, <TokenType.CURRENT_TIME: 'CURRENT_TIME'>, <TokenType.TAG: 'TAG'>, <TokenType.NULL: 'NULL'>, <TokenType.JSONB: 'JSONB'>, <TokenType.USMALLINT: 'USMALLINT'>, <TokenType.TIMESTAMPNTZ: 'TIMESTAMPNTZ'>, <TokenType.INT4MULTIRANGE: 'INT4MULTIRANGE'>, <TokenType.TSTZRANGE: 'TSTZRANGE'>, <TokenType.CURRENT_DATE: 'CURRENT_DATE'>, <TokenType.COLUMN: 'COLUMN'>, <TokenType.DATABASE: 'DATABASE'>, <TokenType.CONSTRAINT: 'CONSTRAINT'>, <TokenType.INT128: 'INT128'>, <TokenType.RECURSIVE: 'RECURSIVE'>, <TokenType.STORAGE_INTEGRATION: 'STORAGE_INTEGRATION'>, <TokenType.SEQUENCE: 'SEQUENCE'>, <TokenType.IPPREFIX: 'IPPREFIX'>}
TABLE_ALIAS_TOKENS = {<TokenType.DATE32: 'DATE32'>, <TokenType.MAP: 'MAP'>, <TokenType.UMEDIUMINT: 'UMEDIUMINT'>, <TokenType.COPY: 'COPY'>, <TokenType.UBIGINT: 'UBIGINT'>, <TokenType.PSEUDO_TYPE: 'PSEUDO_TYPE'>, <TokenType.UNKNOWN: 'UNKNOWN'>, <TokenType.DICTIONARY: 'DICTIONARY'>, <TokenType.UNIQUEIDENTIFIER: 'UNIQUEIDENTIFIER'>, <TokenType.TDIGEST: 'TDIGEST'>, <TokenType.IS: 'IS'>, <TokenType.XML: 'XML'>, <TokenType.VIEW: 'VIEW'>, <TokenType.TIMESTAMPTZ: 'TIMESTAMPTZ'>, <TokenType.ROWVERSION: 'ROWVERSION'>, <TokenType.REFERENCES: 'REFERENCES'>, <TokenType.TOP: 'TOP'>, <TokenType.UUID: 'UUID'>, <TokenType.DIV: 'DIV'>, <TokenType.IPADDRESS: 'IPADDRESS'>, <TokenType.SERIAL: 'SERIAL'>, <TokenType.VARCHAR: 'VARCHAR'>, <TokenType.INT4RANGE: 'INT4RANGE'>, <TokenType.BIT: 'BIT'>, <TokenType.ROW: 'ROW'>, <TokenType.DATETIME: 'DATETIME'>, <TokenType.BPCHAR: 'BPCHAR'>, <TokenType.SEMI: 'SEMI'>, <TokenType.OPERATOR: 'OPERATOR'>, <TokenType.SOME: 'SOME'>, <TokenType.NUMMULTIRANGE: 'NUMMULTIRANGE'>, <TokenType.IMAGE: 'IMAGE'>, <TokenType.CURRENT_DATETIME: 'CURRENT_DATETIME'>, <TokenType.AGGREGATEFUNCTION: 'AGGREGATEFUNCTION'>, <TokenType.TIMESTAMP_NS: 'TIMESTAMP_NS'>, <TokenType.SETTINGS: 'SETTINGS'>, <TokenType.TINYINT: 'TINYINT'>, <TokenType.FOREIGN_KEY: 'FOREIGN_KEY'>, <TokenType.BOOLEAN: 'BOOLEAN'>, <TokenType.BIGDECIMAL: 'BIGDECIMAL'>, <TokenType.DOUBLE: 'DOUBLE'>, <TokenType.MEDIUMBLOB: 'MEDIUMBLOB'>, <TokenType.FALSE: 'FALSE'>, <TokenType.STRAIGHT_JOIN: 'STRAIGHT_JOIN'>, <TokenType.ORDINALITY: 'ORDINALITY'>, <TokenType.UDECIMAL: 'UDECIMAL'>, <TokenType.NEXT: 'NEXT'>, <TokenType.GEOGRAPHY: 'GEOGRAPHY'>, <TokenType.REPLACE: 'REPLACE'>, <TokenType.SEQUENCE: 'SEQUENCE'>, <TokenType.USERDEFINED: 'USERDEFINED'>, <TokenType.SHOW: 'SHOW'>, <TokenType.USE: 'USE'>, <TokenType.FIXEDSTRING: 'FIXEDSTRING'>, <TokenType.TIME: 'TIME'>, <TokenType.ESCAPE: 'ESCAPE'>, <TokenType.COMMAND: 'COMMAND'>, <TokenType.TIMESTAMP: 'TIMESTAMP'>, <TokenType.CURRENT_USER: 'CURRENT_USER'>, <TokenType.MERGE: 'MERGE'>, <TokenType.LONGTEXT: 'LONGTEXT'>, <TokenType.INT8RANGE: 'INT8RANGE'>, <TokenType.ARRAY: 'ARRAY'>, <TokenType.NULLABLE: 'NULLABLE'>, <TokenType.ENUM8: 'ENUM8'>, <TokenType.UTINYINT: 'UTINYINT'>, <TokenType.TIMESTAMP_S: 'TIMESTAMP_S'>, <TokenType.UNNEST: 'UNNEST'>, <TokenType.OBJECT_IDENTIFIER: 'OBJECT_IDENTIFIER'>, <TokenType.UPDATE: 'UPDATE'>, <TokenType.PROCEDURE: 'PROCEDURE'>, <TokenType.ALL: 'ALL'>, <TokenType.OVERWRITE: 'OVERWRITE'>, <TokenType.MODEL: 'MODEL'>, <TokenType.INTERVAL: 'INTERVAL'>, <TokenType.PERCENT: 'PERCENT'>, <TokenType.VECTOR: 'VECTOR'>, <TokenType.OBJECT: 'OBJECT'>, <TokenType.UNIQUE: 'UNIQUE'>, <TokenType.NAME: 'NAME'>, <TokenType.ROLLUP: 'ROLLUP'>, <TokenType.FIRST: 'FIRST'>, <TokenType.TIMESTAMP_MS: 'TIMESTAMP_MS'>, <TokenType.TIMESTAMPLTZ: 'TIMESTAMPLTZ'>, <TokenType.STREAMLIT: 'STREAMLIT'>, <TokenType.LIST: 'LIST'>, <TokenType.RANGE: 'RANGE'>, <TokenType.FORMAT: 'FORMAT'>, <TokenType.YEAR: 'YEAR'>, <TokenType.KEEP: 'KEEP'>, <TokenType.LOAD: 'LOAD'>, <TokenType.PIVOT: 'PIVOT'>, <TokenType.ROWS: 'ROWS'>, <TokenType.SCHEMA: 'SCHEMA'>, <TokenType.EXISTS: 'EXISTS'>, <TokenType.IPV6: 'IPV6'>, <TokenType.UINT: 'UINT'>, <TokenType.INT256: 'INT256'>, <TokenType.TEXT: 'TEXT'>, <TokenType.VARBINARY: 'VARBINARY'>, <TokenType.COLLATE: 'COLLATE'>, <TokenType.BIGINT: 'BIGINT'>, <TokenType.GEOMETRY: 'GEOMETRY'>, <TokenType.VAR: 'VAR'>, <TokenType.CUBE: 'CUBE'>, <TokenType.SET: 'SET'>, <TokenType.SMALLINT: 'SMALLINT'>, <TokenType.IDENTIFIER: 'IDENTIFIER'>, <TokenType.SMALLMONEY: 'SMALLMONEY'>, <TokenType.SMALLSERIAL: 'SMALLSERIAL'>, <TokenType.EXECUTE: 'EXECUTE'>, <TokenType.ASC: 'ASC'>, <TokenType.INET: 'INET'>, <TokenType.NCHAR: 'NCHAR'>, <TokenType.DECIMAL: 'DECIMAL'>, <TokenType.UINT256: 'UINT256'>, <TokenType.WAREHOUSE: 'WAREHOUSE'>, <TokenType.MEDIUMINT: 'MEDIUMINT'>, <TokenType.MEDIUMTEXT: 'MEDIUMTEXT'>, <TokenType.COMMIT: 'COMMIT'>, <TokenType.DELETE: 'DELETE'>, <TokenType.NVARCHAR: 'NVARCHAR'>, <TokenType.INT: 'INT'>, <TokenType.STRUCT: 'STRUCT'>, <TokenType.DEFAULT: 'DEFAULT'>, <TokenType.HLLSKETCH: 'HLLSKETCH'>, <TokenType.INDEX: 'INDEX'>, <TokenType.COMMENT: 'COMMENT'>, <TokenType.BEGIN: 'BEGIN'>, <TokenType.DATETIME64: 'DATETIME64'>, <TokenType.TINYTEXT: 'TINYTEXT'>, <TokenType.FLOAT: 'FLOAT'>, <TokenType.UNPIVOT: 'UNPIVOT'>, <TokenType.TIMETZ: 'TIMETZ'>, <TokenType.TSRANGE: 'TSRANGE'>, <TokenType.MONEY: 'MONEY'>, <TokenType.ENUM16: 'ENUM16'>, <TokenType.HSTORE: 'HSTORE'>, <TokenType.DESCRIBE: 'DESCRIBE'>, <TokenType.TABLE: 'TABLE'>, <TokenType.CURRENT_TIMESTAMP: 'CURRENT_TIMESTAMP'>, <TokenType.REFRESH: 'REFRESH'>, <TokenType.DATE: 'DATE'>, <TokenType.LOWCARDINALITY: 'LOWCARDINALITY'>, <TokenType.END: 'END'>, <TokenType.CACHE: 'CACHE'>, <TokenType.NUMRANGE: 'NUMRANGE'>, <TokenType.ANY: 'ANY'>, <TokenType.TINYBLOB: 'TINYBLOB'>, <TokenType.UINT128: 'UINT128'>, <TokenType.JSON: 'JSON'>, <TokenType.BIGSERIAL: 'BIGSERIAL'>, <TokenType.ANTI: 'ANTI'>, <TokenType.DATEMULTIRANGE: 'DATEMULTIRANGE'>, <TokenType.LONGBLOB: 'LONGBLOB'>, <TokenType.FUNCTION: 'FUNCTION'>, <TokenType.DESC: 'DESC'>, <TokenType.ISNULL: 'ISNULL'>, <TokenType.FINAL: 'FINAL'>, <TokenType.TRUNCATE: 'TRUNCATE'>, <TokenType.OVERLAPS: 'OVERLAPS'>, <TokenType.TSMULTIRANGE: 'TSMULTIRANGE'>, <TokenType.FILTER: 'FILTER'>, <TokenType.PRAGMA: 'PRAGMA'>, <TokenType.TRUE: 'TRUE'>, <TokenType.INT8MULTIRANGE: 'INT8MULTIRANGE'>, <TokenType.CHAR: 'CHAR'>, <TokenType.SIMPLEAGGREGATEFUNCTION: 'SIMPLEAGGREGATEFUNCTION'>, <TokenType.VARIANT: 'VARIANT'>, <TokenType.SUPER: 'SUPER'>, <TokenType.TEMPORARY: 'TEMPORARY'>, <TokenType.CASE: 'CASE'>, <TokenType.AUTO_INCREMENT: 'AUTO_INCREMENT'>, <TokenType.IPV4: 'IPV4'>, <TokenType.KILL: 'KILL'>, <TokenType.NESTED: 'NESTED'>, <TokenType.BINARY: 'BINARY'>, <TokenType.VOLATILE: 'VOLATILE'>, <TokenType.PARTITION: 'PARTITION'>, <TokenType.DATERANGE: 'DATERANGE'>, <TokenType.TSTZMULTIRANGE: 'TSTZMULTIRANGE'>, <TokenType.CURRENT_TIME: 'CURRENT_TIME'>, <TokenType.TAG: 'TAG'>, <TokenType.NULL: 'NULL'>, <TokenType.JSONB: 'JSONB'>, <TokenType.USMALLINT: 'USMALLINT'>, <TokenType.TIMESTAMPNTZ: 'TIMESTAMPNTZ'>, <TokenType.INT4MULTIRANGE: 'INT4MULTIRANGE'>, <TokenType.TSTZRANGE: 'TSTZRANGE'>, <TokenType.CURRENT_DATE: 'CURRENT_DATE'>, <TokenType.COLUMN: 'COLUMN'>, <TokenType.DATABASE: 'DATABASE'>, <TokenType.CONSTRAINT: 'CONSTRAINT'>, <TokenType.INT128: 'INT128'>, <TokenType.RECURSIVE: 'RECURSIVE'>, <TokenType.STORAGE_INTEGRATION: 'STORAGE_INTEGRATION'>, <TokenType.ENUM: 'ENUM'>, <TokenType.IPPREFIX: 'IPPREFIX'>}
SHOW_TRIE: Dict = {}
SET_TRIE: Dict = {'GLOBAL': {0: True}, 'LOCAL': {0: True}, 'SESSION': {0: True}, 'TRANSACTION': {0: True}}

##### Inherited Members

sqlglot.parser.Parser
  Parser
  NO_PAREN_FUNCTIONS
  STRUCT_TYPE_TOKENS
  NESTED_TYPE_TOKENS
  ENUM_TYPE_TOKENS
  AGGREGATE_TYPE_TOKENS
  TYPE_TOKENS
  SIGNED_TO_UNSIGNED_TYPE_TOKEN
  SUBQUERY_PREDICATES
  RESERVED_TOKENS
  DB_CREATABLES
  CREATABLES
  INTERVAL_VARS
  ALIAS_TOKENS
  ARRAY_CONSTRUCTORS
  COMMENT_TABLE_ALIAS_TOKENS
  UPDATE_ALIAS_TOKENS
  TRIM_TYPES
  FUNC_TOKENS
  CONJUNCTION
  ASSIGNMENT
  DISJUNCTION
  EQUALITY
  COMPARISON
  BITWISE
  TERM
  FACTOR
  EXPONENT
  TIMES
  TIMESTAMPS
  SET_OPERATIONS
  JOIN_METHODS
  JOIN_SIDES
  JOIN_KINDS
  JOIN_HINTS
  LAMBDAS
  COLUMN_OPERATORS
  EXPRESSION_PARSERS
  STATEMENT_PARSERS
  UNARY_PARSERS
  STRING_PARSERS
  NUMERIC_PARSERS
  PRIMARY_PARSERS
  PLACEHOLDER_PARSERS
  RANGE_PARSERS
  PROPERTY_PARSERS
  CONSTRAINT_PARSERS
  ALTER_PARSERS
  ALTER_ALTER_PARSERS
  SCHEMA_UNNAMED_CONSTRAINTS
  NO_PAREN_FUNCTION_PARSERS
  INVALID_FUNC_NAME_TOKENS
  FUNCTIONS_WITH_ALIASED_ARGS
  KEY_VALUE_DEFINITIONS
  QUERY_MODIFIER_PARSERS
  SET_PARSERS
  SHOW_PARSERS
  TYPE_LITERAL_PARSERS
  TYPE_CONVERTERS
  DDL_SELECT_TOKENS
  PRE_VOLATILE_TOKENS
  TRANSACTION_KIND
  TRANSACTION_CHARACTERISTICS
  CONFLICT_ACTIONS
  CREATE_SEQUENCE
  ISOLATED_LOADING_OPTIONS
  USABLES
  CAST_ACTIONS
  SCHEMA_BINDING_OPTIONS
  KEY_CONSTRAINT_OPTIONS
  INSERT_ALTERNATIVES
  CLONE_KEYWORDS
  HISTORICAL_DATA_PREFIX
  HISTORICAL_DATA_KIND
  OPCLASS_FOLLOW_KEYWORDS
  OPTYPE_FOLLOW_TOKENS
  TABLE_INDEX_HINT_TOKENS
  VIEW_ATTRIBUTES
  WINDOW_ALIAS_TOKENS
  WINDOW_BEFORE_PAREN_TOKENS
  WINDOW_SIDES
  JSON_KEY_VALUE_SEPARATOR_TOKENS
  FETCH_TOKENS
  ADD_CONSTRAINT_TOKENS
  DISTINCT_TOKENS
  NULL_TOKENS
  UNNEST_OFFSET_ALIAS_TOKENS
  SELECT_START_TOKENS
  COPY_INTO_VARLEN_OPTIONS
  STRICT_CAST
  PREFIXED_PIVOT_COLUMNS
  IDENTIFY_PIVOT_STRINGS
  LOG_DEFAULTS_TO_LN
  ALTER_TABLE_ADD_REQUIRED_FOR_EACH_COLUMN
  TABLESAMPLE_CSV
  DEFAULT_SAMPLING_METHOD
  SET_REQUIRES_ASSIGNMENT_DELIMITER
  TRIM_PATTERN_FIRST
  STRING_ALIASES
  MODIFIERS_ATTACHED_TO_SET_OP
  SET_OP_MODIFIERS
  NO_PAREN_IF_COMMANDS
  JSON_ARROWS_REQUIRE_JSON_TYPE
  COLON_IS_VARIANT_EXTRACT
  SUPPORTS_IMPLICIT_UNNEST
  INTERVAL_SPANS
  SUPPORTS_PARTITION_SELECTION
  error_level
  error_message_context
  max_errors
  dialect
  reset
  parse
  parse_into
  check_errors
  raise_error
  expression
  validate_expression
  errors
  sql

class Presto.Generator(sqlglot.generator.Generator): View Source

    356    class Generator(generator.Generator):
    357        INTERVAL_ALLOWS_PLURAL_FORM = False
    358        JOIN_HINTS = False
    359        TABLE_HINTS = False
    360        QUERY_HINTS = False
    361        IS_BOOL_ALLOWED = False
    362        TZ_TO_WITH_TIME_ZONE = True
    363        NVL2_SUPPORTED = False
    364        STRUCT_DELIMITER = ("(", ")")
    365        LIMIT_ONLY_LITERALS = True
    366        SUPPORTS_SINGLE_ARG_CONCAT = False
    367        LIKE_PROPERTY_INSIDE_SCHEMA = True
    368        MULTI_ARG_DISTINCT = False
    369        SUPPORTS_TO_NUMBER = False
    370        HEX_FUNC = "TO_HEX"
    371        PARSE_JSON_NAME = "JSON_PARSE"
    372        PAD_FILL_PATTERN_IS_REQUIRED = True
    373
    374        PROPERTIES_LOCATION = {
    375            **generator.Generator.PROPERTIES_LOCATION,
    376            exp.LocationProperty: exp.Properties.Location.UNSUPPORTED,
    377            exp.VolatileProperty: exp.Properties.Location.UNSUPPORTED,
    378        }
    379
    380        TYPE_MAPPING = {
    381            **generator.Generator.TYPE_MAPPING,
    382            exp.DataType.Type.INT: "INTEGER",
    383            exp.DataType.Type.FLOAT: "REAL",
    384            exp.DataType.Type.BINARY: "VARBINARY",
    385            exp.DataType.Type.TEXT: "VARCHAR",
    386            exp.DataType.Type.TIMETZ: "TIME",
    387            exp.DataType.Type.TIMESTAMPTZ: "TIMESTAMP",
    388            exp.DataType.Type.STRUCT: "ROW",
    389            exp.DataType.Type.DATETIME: "TIMESTAMP",
    390            exp.DataType.Type.DATETIME64: "TIMESTAMP",
    391            exp.DataType.Type.HLLSKETCH: "HYPERLOGLOG",
    392        }
    393
    394        TRANSFORMS = {
    395            **generator.Generator.TRANSFORMS,
    396            exp.AnyValue: rename_func("ARBITRARY"),
    397            exp.ApproxQuantile: rename_func("APPROX_PERCENTILE"),
    398            exp.ArgMax: rename_func("MAX_BY"),
    399            exp.ArgMin: rename_func("MIN_BY"),
    400            exp.Array: lambda self, e: f"ARRAY[{self.expressions(e, flat=True)}]",
    401            exp.ArrayAny: rename_func("ANY_MATCH"),
    402            exp.ArrayConcat: rename_func("CONCAT"),
    403            exp.ArrayContains: rename_func("CONTAINS"),
    404            exp.ArraySize: rename_func("CARDINALITY"),
    405            exp.ArrayToString: rename_func("ARRAY_JOIN"),
    406            exp.ArrayUniqueAgg: rename_func("SET_AGG"),
    407            exp.AtTimeZone: rename_func("AT_TIMEZONE"),
    408            exp.BitwiseAnd: lambda self, e: self.func("BITWISE_AND", e.this, e.expression),
    409            exp.BitwiseLeftShift: lambda self, e: self.func(
    410                "BITWISE_ARITHMETIC_SHIFT_LEFT", e.this, e.expression
    411            ),
    412            exp.BitwiseNot: lambda self, e: self.func("BITWISE_NOT", e.this),
    413            exp.BitwiseOr: lambda self, e: self.func("BITWISE_OR", e.this, e.expression),
    414            exp.BitwiseRightShift: lambda self, e: self.func(
    415                "BITWISE_ARITHMETIC_SHIFT_RIGHT", e.this, e.expression
    416            ),
    417            exp.BitwiseXor: lambda self, e: self.func("BITWISE_XOR", e.this, e.expression),
    418            exp.Cast: transforms.preprocess([transforms.epoch_cast_to_ts]),
    419            exp.CurrentTimestamp: lambda *_: "CURRENT_TIMESTAMP",
    420            exp.DateAdd: _date_delta_sql("DATE_ADD"),
    421            exp.DateDiff: lambda self, e: self.func(
    422                "DATE_DIFF", unit_to_str(e), e.expression, e.this
    423            ),
    424            exp.DateStrToDate: datestrtodate_sql,
    425            exp.DateToDi: lambda self,
    426            e: f"CAST(DATE_FORMAT({self.sql(e, 'this')}, {Presto.DATEINT_FORMAT}) AS INT)",
    427            exp.DateSub: _date_delta_sql("DATE_ADD", negate_interval=True),
    428            exp.Decode: lambda self, e: encode_decode_sql(self, e, "FROM_UTF8"),
    429            exp.DiToDate: lambda self,
    430            e: f"CAST(DATE_PARSE(CAST({self.sql(e, 'this')} AS VARCHAR), {Presto.DATEINT_FORMAT}) AS DATE)",
    431            exp.Encode: lambda self, e: encode_decode_sql(self, e, "TO_UTF8"),
    432            exp.FileFormatProperty: lambda self, e: f"FORMAT='{e.name.upper()}'",
    433            exp.First: _first_last_sql,
    434            exp.FirstValue: _first_last_sql,
    435            exp.FromTimeZone: lambda self,
    436            e: f"WITH_TIMEZONE({self.sql(e, 'this')}, {self.sql(e, 'zone')}) AT TIME ZONE 'UTC'",
    437            exp.GenerateSeries: sequence_sql,
    438            exp.Group: transforms.preprocess([transforms.unalias_group]),
    439            exp.GroupConcat: lambda self, e: self.func(
    440                "ARRAY_JOIN", self.func("ARRAY_AGG", e.this), e.args.get("separator")
    441            ),
    442            exp.If: if_sql(),
    443            exp.ILike: no_ilike_sql,
    444            exp.Initcap: _initcap_sql,
    445            exp.JSONExtract: _jsonextract_sql,
    446            exp.Last: _first_last_sql,
    447            exp.LastValue: _first_last_sql,
    448            exp.LastDay: lambda self, e: self.func("LAST_DAY_OF_MONTH", e.this),
    449            exp.Lateral: _explode_to_unnest_sql,
    450            exp.Left: left_to_substring_sql,
    451            exp.Levenshtein: rename_func("LEVENSHTEIN_DISTANCE"),
    452            exp.LogicalAnd: rename_func("BOOL_AND"),
    453            exp.LogicalOr: rename_func("BOOL_OR"),
    454            exp.Pivot: no_pivot_sql,
    455            exp.Quantile: _quantile_sql,
    456            exp.RegexpExtract: regexp_extract_sql,
    457            exp.Right: right_to_substring_sql,
    458            exp.SafeDivide: no_safe_divide_sql,
    459            exp.Schema: _schema_sql,
    460            exp.SchemaCommentProperty: lambda self, e: self.naked_property(e),
    461            exp.Select: transforms.preprocess(
    462                [
    463                    transforms.eliminate_qualify,
    464                    transforms.eliminate_distinct_on,
    465                    transforms.explode_to_unnest(1),
    466                    transforms.eliminate_semi_and_anti_joins,
    467                ]
    468            ),
    469            exp.SortArray: _no_sort_array,
    470            exp.StrPosition: lambda self, e: str_position_sql(self, e, generate_instance=True),
    471            exp.StrToDate: lambda self, e: f"CAST({_str_to_time_sql(self, e)} AS DATE)",
    472            exp.StrToMap: rename_func("SPLIT_TO_MAP"),
    473            exp.StrToTime: _str_to_time_sql,
    474            exp.StructExtract: struct_extract_sql,
    475            exp.Table: transforms.preprocess([_unnest_sequence]),
    476            exp.Timestamp: no_timestamp_sql,
    477            exp.TimestampAdd: _date_delta_sql("DATE_ADD"),
    478            exp.TimestampTrunc: timestamptrunc_sql(),
    479            exp.TimeStrToDate: timestrtotime_sql,
    480            exp.TimeStrToTime: timestrtotime_sql,
    481            exp.TimeStrToUnix: lambda self, e: self.func(
    482                "TO_UNIXTIME", self.func("DATE_PARSE", e.this, Presto.TIME_FORMAT)
    483            ),
    484            exp.TimeToStr: lambda self, e: self.func("DATE_FORMAT", e.this, self.format_time(e)),
    485            exp.TimeToUnix: rename_func("TO_UNIXTIME"),
    486            exp.ToChar: lambda self, e: self.func("DATE_FORMAT", e.this, self.format_time(e)),
    487            exp.TryCast: transforms.preprocess([transforms.epoch_cast_to_ts]),
    488            exp.TsOrDiToDi: lambda self,
    489            e: f"CAST(SUBSTR(REPLACE(CAST({self.sql(e, 'this')} AS VARCHAR), '-', ''), 1, 8) AS INT)",
    490            exp.TsOrDsAdd: _ts_or_ds_add_sql,
    491            exp.TsOrDsDiff: _ts_or_ds_diff_sql,
    492            exp.TsOrDsToDate: _ts_or_ds_to_date_sql,
    493            exp.Unhex: rename_func("FROM_HEX"),
    494            exp.UnixToStr: lambda self,
    495            e: f"DATE_FORMAT(FROM_UNIXTIME({self.sql(e, 'this')}), {self.format_time(e)})",
    496            exp.UnixToTime: _unix_to_time_sql,
    497            exp.UnixToTimeStr: lambda self,
    498            e: f"CAST(FROM_UNIXTIME({self.sql(e, 'this')}) AS VARCHAR)",
    499            exp.VariancePop: rename_func("VAR_POP"),
    500            exp.With: transforms.preprocess([transforms.add_recursive_cte_column_names]),
    501            exp.WithinGroup: transforms.preprocess(
    502                [transforms.remove_within_group_for_percentiles]
    503            ),
    504            exp.Xor: bool_xor_sql,
    505            exp.MD5Digest: rename_func("MD5"),
    506            exp.SHA: rename_func("SHA1"),
    507            exp.SHA2: sha256_sql,
    508        }
    509
    510        RESERVED_KEYWORDS = {
    511            "alter",
    512            "and",
    513            "as",
    514            "between",
    515            "by",
    516            "case",
    517            "cast",
    518            "constraint",
    519            "create",
    520            "cross",
    521            "current_time",
    522            "current_timestamp",
    523            "deallocate",
    524            "delete",
    525            "describe",
    526            "distinct",
    527            "drop",
    528            "else",
    529            "end",
    530            "escape",
    531            "except",
    532            "execute",
    533            "exists",
    534            "extract",
    535            "false",
    536            "for",
    537            "from",
    538            "full",
    539            "group",
    540            "having",
    541            "in",
    542            "inner",
    543            "insert",
    544            "intersect",
    545            "into",
    546            "is",
    547            "join",
    548            "left",
    549            "like",
    550            "natural",
    551            "not",
    552            "null",
    553            "on",
    554            "or",
    555            "order",
    556            "outer",
    557            "prepare",
    558            "right",
    559            "select",
    560            "table",
    561            "then",
    562            "true",
    563            "union",
    564            "using",
    565            "values",
    566            "when",
    567            "where",
    568            "with",
    569        }
    570
    571        def md5_sql(self, expression: exp.MD5) -> str:
    572            this = expression.this
    573
    574            if not this.type:
    575                from sqlglot.optimizer.annotate_types import annotate_types
    576
    577                this = annotate_types(this)
    578
    579            if this.is_type(*exp.DataType.TEXT_TYPES):
    580                this = exp.Encode(this=this, charset=exp.Literal.string("utf-8"))
    581
    582            return self.func("LOWER", self.func("TO_HEX", self.func("MD5", self.sql(this))))
    583
    584        def strtounix_sql(self, expression: exp.StrToUnix) -> str:
    585            # Since `TO_UNIXTIME` requires a `TIMESTAMP`, we need to parse the argument into one.
    586            # To do this, we first try to `DATE_PARSE` it, but since this can fail when there's a
    587            # timezone involved, we wrap it in a `TRY` call and use `PARSE_DATETIME` as a fallback,
    588            # which seems to be using the same time mapping as Hive, as per:
    589            # https://joda-time.sourceforge.net/apidocs/org/joda/time/format/DateTimeFormat.html
    590            this = expression.this
    591            value_as_text = exp.cast(this, exp.DataType.Type.TEXT)
    592            value_as_timestamp = (
    593                exp.cast(this, exp.DataType.Type.TIMESTAMP) if this.is_string else this
    594            )
    595
    596            parse_without_tz = self.func("DATE_PARSE", value_as_text, self.format_time(expression))
    597
    598            formatted_value = self.func(
    599                "DATE_FORMAT", value_as_timestamp, self.format_time(expression)
    600            )
    601            parse_with_tz = self.func(
    602                "PARSE_DATETIME",
    603                formatted_value,
    604                self.format_time(expression, Hive.INVERSE_TIME_MAPPING, Hive.INVERSE_TIME_TRIE),
    605            )
    606            coalesced = self.func("COALESCE", self.func("TRY", parse_without_tz), parse_with_tz)
    607            return self.func("TO_UNIXTIME", coalesced)
    608
    609        def bracket_sql(self, expression: exp.Bracket) -> str:
    610            if expression.args.get("safe"):
    611                return self.func(
    612                    "ELEMENT_AT",
    613                    expression.this,
    614                    seq_get(
    615                        apply_index_offset(
    616                            expression.this,
    617                            expression.expressions,
    618                            1 - expression.args.get("offset", 0),
    619                        ),
    620                        0,
    621                    ),
    622                )
    623            return super().bracket_sql(expression)
    624
    625        def struct_sql(self, expression: exp.Struct) -> str:
    626            from sqlglot.optimizer.annotate_types import annotate_types
    627
    628            expression = annotate_types(expression)
    629            values: t.List[str] = []
    630            schema: t.List[str] = []
    631            unknown_type = False
    632
    633            for e in expression.expressions:
    634                if isinstance(e, exp.PropertyEQ):
    635                    if e.type and e.type.is_type(exp.DataType.Type.UNKNOWN):
    636                        unknown_type = True
    637                    else:
    638                        schema.append(f"{self.sql(e, 'this')} {self.sql(e.type)}")
    639                    values.append(self.sql(e, "expression"))
    640                else:
    641                    values.append(self.sql(e))
    642
    643            size = len(expression.expressions)
    644
    645            if not size or len(schema) != size:
    646                if unknown_type:
    647                    self.unsupported(
    648                        "Cannot convert untyped key-value definitions (try annotate_types)."
    649                    )
    650                return self.func("ROW", *values)
    651            return f"CAST(ROW({', '.join(values)}) AS ROW({', '.join(schema)}))"
    652
    653        def interval_sql(self, expression: exp.Interval) -> str:
    654            if expression.this and expression.text("unit").upper().startswith("WEEK"):
    655                return f"({expression.this.name} * INTERVAL '7' DAY)"
    656            return super().interval_sql(expression)
    657
    658        def transaction_sql(self, expression: exp.Transaction) -> str:
    659            modes = expression.args.get("modes")
    660            modes = f" {', '.join(modes)}" if modes else ""
    661            return f"START TRANSACTION{modes}"
    662
    663        def offset_limit_modifiers(
    664            self, expression: exp.Expression, fetch: bool, limit: t.Optional[exp.Fetch | exp.Limit]
    665        ) -> t.List[str]:
    666            return [
    667                self.sql(expression, "offset"),
    668                self.sql(limit),
    669            ]
    670
    671        def create_sql(self, expression: exp.Create) -> str:
    672            """
    673            Presto doesn't support CREATE VIEW with expressions (ex: `CREATE VIEW x (cola)` then `(cola)` is the expression),
    674            so we need to remove them
    675            """
    676            kind = expression.args["kind"]
    677            schema = expression.this
    678            if kind == "VIEW" and schema.expressions:
    679                expression.this.set("expressions", None)
    680            return super().create_sql(expression)
    681
    682        def delete_sql(self, expression: exp.Delete) -> str:
    683            """
    684            Presto only supports DELETE FROM for a single table without an alias, so we need
    685            to remove the unnecessary parts. If the original DELETE statement contains more
    686            than one table to be deleted, we can't safely map it 1-1 to a Presto statement.
    687            """
    688            tables = expression.args.get("tables") or [expression.this]
    689            if len(tables) > 1:
    690                return super().delete_sql(expression)
    691
    692            table = tables[0]
    693            expression.set("this", table)
    694            expression.set("tables", None)
    695
    696            if isinstance(table, exp.Table):
    697                table_alias = table.args.get("alias")
    698                if table_alias:
    699                    table_alias.pop()
    700                    expression = t.cast(exp.Delete, expression.transform(unqualify_columns))
    701
    702            return super().delete_sql(expression)

Generator converts a given syntax tree to the corresponding SQL string.

###### Arguments:

- pretty: Whether to format the produced SQL string. Default: False.
- identify: Determines when an identifier should be quoted. Possible values are: False (default): Never quote, except in cases where it's mandatory by the dialect. True or 'always': Always quote. 'safe': Only quote identifiers that are case insensitive.
- normalize: Whether to normalize identifiers to lowercase. Default: False.
- pad: The pad size in a formatted string. For example, this affects the indentation of a projection in a query, relative to its nesting level. Default: 2.
- indent: The indentation size in a formatted string. For example, this affects the indentation of subqueries and filters under a `WHERE` clause. Default: 2.
- normalize_functions: How to normalize function names. Possible values are: "upper" or True (default): Convert names to uppercase. "lower": Convert names to lowercase. False: Disables function name normalization.
- unsupported_level: Determines the generator's behavior when it encounters unsupported expressions. Default ErrorLevel.WARN.
- max_unsupported: Maximum number of unsupported messages to include in a raised UnsupportedError. This is only relevant if unsupported_level is ErrorLevel.RAISE. Default: 3
- leading_comma: Whether the comma is leading or trailing in select expressions. This is only relevant when generating in pretty mode. Default: False
- max_text_width: The max number of characters in a segment before creating new lines in pretty mode. The default is on the smaller end because the length only represents a segment and not the true line length. Default: 80
- comments: Whether to preserve comments in the output SQL code. Default: True

INTERVAL_ALLOWS_PLURAL_FORM = False
JOIN_HINTS = False
TABLE_HINTS = False
QUERY_HINTS = False
IS_BOOL_ALLOWED = False
TZ_TO_WITH_TIME_ZONE = True
NVL2_SUPPORTED = False
STRUCT_DELIMITER = ('(', ')')
LIMIT_ONLY_LITERALS = True
SUPPORTS_SINGLE_ARG_CONCAT = False
LIKE_PROPERTY_INSIDE_SCHEMA = True
MULTI_ARG_DISTINCT = False
SUPPORTS_TO_NUMBER = False
HEX_FUNC = 'TO_HEX'
PARSE_JSON_NAME = 'JSON_PARSE'
PAD_FILL_PATTERN_IS_REQUIRED = True
PROPERTIES_LOCATION = {<class 'sqlglot.expressions.AllowedValuesProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.AlgorithmProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.AutoIncrementProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.AutoRefreshProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.BackupProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.BlockCompressionProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.CharacterSetProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ChecksumProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.CollateProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.CopyGrantsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.Cluster'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ClusteredByProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DataBlocksizeProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.DataDeletionProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DefinerProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.DictRange'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DictProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DynamicProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.DistKeyProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DistStyleProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.EngineProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ExecuteAsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ExternalProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.FallbackProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.FileFormatProperty'>: <Location.POST_WITH: 'POST_WITH'>, <class 'sqlglot.expressions.FreespaceProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.GlobalProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.HeapProperty'>: <Location.POST_WITH: 'POST_WITH'>, <class 'sqlglot.expressions.InheritsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.IcebergProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.InputModelProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.IsolatedLoadingProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.JournalProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.LanguageProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.LikeProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.LocationProperty'>: <Location.UNSUPPORTED: 'UNSUPPORTED'>, <class 'sqlglot.expressions.LockProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.LockingProperty'>: <Location.POST_ALIAS: 'POST_ALIAS'>, <class 'sqlglot.expressions.LogProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.MaterializedProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.MergeBlockRatioProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.NoPrimaryIndexProperty'>: <Location.POST_EXPRESSION: 'POST_EXPRESSION'>, <class 'sqlglot.expressions.OnProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.OnCommitProperty'>: <Location.POST_EXPRESSION: 'POST_EXPRESSION'>, <class 'sqlglot.expressions.Order'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.OutputModelProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.PartitionedByProperty'>: <Location.POST_WITH: 'POST_WITH'>, <class 'sqlglot.expressions.PartitionedOfProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.PrimaryKey'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.Property'>: <Location.POST_WITH: 'POST_WITH'>, <class 'sqlglot.expressions.RemoteWithConnectionModelProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ReturnsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.RowFormatProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.RowFormatDelimitedProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.RowFormatSerdeProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SampleProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SchemaCommentProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SecureProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.SerdeProperties'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.Set'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SettingsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SetProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.SetConfigProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SharingProperty'>: <Location.POST_EXPRESSION: 'POST_EXPRESSION'>, <class 'sqlglot.expressions.SequenceProperties'>: <Location.POST_EXPRESSION: 'POST_EXPRESSION'>, <class 'sqlglot.expressions.SortKeyProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SqlReadWriteProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SqlSecurityProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.StabilityProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.StrictProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.TemporaryProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.ToTableProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.TransientProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.TransformModelProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.MergeTreeTTL'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.UnloggedProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.ViewAttributeProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.VolatileProperty'>: <Location.UNSUPPORTED: 'UNSUPPORTED'>, <class 'sqlglot.expressions.WithDataProperty'>: <Location.POST_EXPRESSION: 'POST_EXPRESSION'>, <class 'sqlglot.expressions.WithJournalTableProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.WithSchemaBindingProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.WithSystemVersioningProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>}
TYPE_MAPPING = {<Type.NCHAR: 'NCHAR'>: 'CHAR', <Type.NVARCHAR: 'NVARCHAR'>: 'VARCHAR', <Type.MEDIUMTEXT: 'MEDIUMTEXT'>: 'TEXT', <Type.LONGTEXT: 'LONGTEXT'>: 'TEXT', <Type.TINYTEXT: 'TINYTEXT'>: 'TEXT', <Type.MEDIUMBLOB: 'MEDIUMBLOB'>: 'BLOB', <Type.LONGBLOB: 'LONGBLOB'>: 'BLOB', <Type.TINYBLOB: 'TINYBLOB'>: 'BLOB', <Type.INET: 'INET'>: 'INET', <Type.ROWVERSION: 'ROWVERSION'>: 'VARBINARY', <Type.INT: 'INT'>: 'INTEGER', <Type.FLOAT: 'FLOAT'>: 'REAL', <Type.BINARY: 'BINARY'>: 'VARBINARY', <Type.TEXT: 'TEXT'>: 'VARCHAR', <Type.TIMETZ: 'TIMETZ'>: 'TIME', <Type.TIMESTAMPTZ: 'TIMESTAMPTZ'>: 'TIMESTAMP', <Type.STRUCT: 'STRUCT'>: 'ROW', <Type.DATETIME: 'DATETIME'>: 'TIMESTAMP', <Type.DATETIME64: 'DATETIME64'>: 'TIMESTAMP', <Type.HLLSKETCH: 'HLLSKETCH'>: 'HYPERLOGLOG'}
TRANSFORMS = {<class 'sqlglot.expressions.JSONPathFilter'>: <function <lambda>>, <class 'sqlglot.expressions.JSONPathKey'>: <function <lambda>>, <class 'sqlglot.expressions.JSONPathRecursive'>: <function <lambda>>, <class 'sqlglot.expressions.JSONPathRoot'>: <function <lambda>>, <class 'sqlglot.expressions.JSONPathScript'>: <function <lambda>>, <class 'sqlglot.expressions.JSONPathSelector'>: <function <lambda>>, <class 'sqlglot.expressions.JSONPathSlice'>: <function <lambda>>, <class 'sqlglot.expressions.JSONPathSubscript'>: <function <lambda>>, <class 'sqlglot.expressions.JSONPathUnion'>: <function <lambda>>, <class 'sqlglot.expressions.JSONPathWildcard'>: <function <lambda>>, <class 'sqlglot.expressions.AllowedValuesProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.AutoRefreshProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.BackupProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CaseSpecificColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CharacterSetColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CharacterSetProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ClusteredColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CollateColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CommentColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ConnectByRoot'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CopyGrantsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.DateFormatColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.DefaultColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.DynamicProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.EncodeColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.EphemeralColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ExcludeColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ExecuteAsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ExternalProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.GlobalProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.HeapProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.IcebergProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.InheritsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.InlineLengthColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.InputModelProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.IntervalSpan'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.LanguageProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.LocationProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.LogProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.MaterializedProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.NonClusteredColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.NoPrimaryIndexProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.NotForReplicationColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.OnCommitProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.OnProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.OnUpdateColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.OutputModelProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.PathColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ProjectionPolicyColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.RemoteWithConnectionModelProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ReturnsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SampleProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SecureProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SetConfigProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SetProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SettingsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SharingProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SqlReadWriteProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SqlSecurityProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.StabilityProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.StrictProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.TemporaryProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.TagColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.TitleColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ToMap'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ToTableProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.TransformModelProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.TransientProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.UppercaseColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.UnloggedProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.VarMap'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ViewAttributeProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.VolatileProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.WithJournalTableProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.WithSchemaBindingProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.WithOperator'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.AnyValue'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.ApproxQuantile'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.ArgMax'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.ArgMin'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.Array'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.ArrayAny'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.ArrayConcat'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.ArrayContains'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.ArraySize'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.ArrayToString'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.ArrayUniqueAgg'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.AtTimeZone'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.BitwiseAnd'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.BitwiseLeftShift'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.BitwiseNot'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.BitwiseOr'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.BitwiseRightShift'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.BitwiseXor'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.Cast'>: <function preprocess.<locals>._to_sql>, <class 'sqlglot.expressions.CurrentTimestamp'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.DateAdd'>: <function _date_delta_sql.<locals>._delta_sql>, <class 'sqlglot.expressions.DateDiff'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.DateStrToDate'>: <function datestrtodate_sql>, <class 'sqlglot.expressions.DateToDi'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.DateSub'>: <function _date_delta_sql.<locals>._delta_sql>, <class 'sqlglot.expressions.Decode'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.DiToDate'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.Encode'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.FileFormatProperty'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.First'>: <function _first_last_sql>, <class 'sqlglot.expressions.FirstValue'>: <function _first_last_sql>, <class 'sqlglot.expressions.FromTimeZone'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.GenerateSeries'>: <function sequence_sql>, <class 'sqlglot.expressions.Group'>: <function preprocess.<locals>._to_sql>, <class 'sqlglot.expressions.GroupConcat'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.If'>: <function if_sql.<locals>._if_sql>, <class 'sqlglot.expressions.ILike'>: <function no_ilike_sql>, <class 'sqlglot.expressions.Initcap'>: <function _initcap_sql>, <class 'sqlglot.expressions.JSONExtract'>: <function _jsonextract_sql>, <class 'sqlglot.expressions.Last'>: <function _first_last_sql>, <class 'sqlglot.expressions.LastValue'>: <function _first_last_sql>, <class 'sqlglot.expressions.LastDay'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.Lateral'>: <function _explode_to_unnest_sql>, <class 'sqlglot.expressions.Left'>: <function left_to_substring_sql>, <class 'sqlglot.expressions.Levenshtein'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.LogicalAnd'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.LogicalOr'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.Pivot'>: <function no_pivot_sql>, <class 'sqlglot.expressions.Quantile'>: <function _quantile_sql>, <class 'sqlglot.expressions.RegexpExtract'>: <function regexp_extract_sql>, <class 'sqlglot.expressions.Right'>: <function right_to_substring_sql>, <class 'sqlglot.expressions.SafeDivide'>: <function no_safe_divide_sql>, <class 'sqlglot.expressions.Schema'>: <function _schema_sql>, <class 'sqlglot.expressions.SchemaCommentProperty'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.Select'>: <function preprocess.<locals>._to_sql>, <class 'sqlglot.expressions.SortArray'>: <function _no_sort_array>, <class 'sqlglot.expressions.StrPosition'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.StrToDate'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.StrToMap'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.StrToTime'>: <function _str_to_time_sql>, <class 'sqlglot.expressions.StructExtract'>: <function struct_extract_sql>, <class 'sqlglot.expressions.Table'>: <function preprocess.<locals>._to_sql>, <class 'sqlglot.expressions.Timestamp'>: <function no_timestamp_sql>, <class 'sqlglot.expressions.TimestampAdd'>: <function _date_delta_sql.<locals>._delta_sql>, <class 'sqlglot.expressions.TimestampTrunc'>: <function timestamptrunc_sql.<locals>._timestamptrunc_sql>, <class 'sqlglot.expressions.TimeStrToDate'>: <function timestrtotime_sql>, <class 'sqlglot.expressions.TimeStrToTime'>: <function timestrtotime_sql>, <class 'sqlglot.expressions.TimeStrToUnix'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.TimeToStr'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.TimeToUnix'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.ToChar'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.TryCast'>: <function preprocess.<locals>._to_sql>, <class 'sqlglot.expressions.TsOrDiToDi'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.TsOrDsAdd'>: <function _ts_or_ds_add_sql>, <class 'sqlglot.expressions.TsOrDsDiff'>: <function _ts_or_ds_diff_sql>, <class 'sqlglot.expressions.TsOrDsToDate'>: <function _ts_or_ds_to_date_sql>, <class 'sqlglot.expressions.Unhex'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.UnixToStr'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.UnixToTime'>: <function _unix_to_time_sql>, <class 'sqlglot.expressions.UnixToTimeStr'>: <function Presto.Generator.<lambda>>, <class 'sqlglot.expressions.VariancePop'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.With'>: <function preprocess.<locals>._to_sql>, <class 'sqlglot.expressions.WithinGroup'>: <function preprocess.<locals>._to_sql>, <class 'sqlglot.expressions.Xor'>: <function bool_xor_sql>, <class 'sqlglot.expressions.MD5Digest'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.SHA'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.SHA2'>: <function sha256_sql>}
RESERVED_KEYWORDS = {'group', 'current_time', 'cross', 'join', 'execute', 'order', 'values', 'drop', 'having', 'like', 'deallocate', 'except', 'between', 'by', 'outer', 'full', 'then', 'as', 'when', 'right', 'delete', 'inner', 'end', 'into', 'create', 'intersect', 'table', 'extract', 'select', 'alter', 'distinct', 'true', 'and', 'false', 'where', 'in', 'prepare', 'natural', 'on', 'is', 'case', 'with', 'null', 'insert', 'current_timestamp', 'union', 'for', 'not', 'or', 'describe', 'exists', 'else', 'using', 'constraint', 'from', 'escape', 'left', 'cast'}
def md5_sql(self, expression: sqlglot.expressions.MD5) -> str: View Source

    571        def md5_sql(self, expression: exp.MD5) -> str:
    572            this = expression.this
    573
    574            if not this.type:
    575                from sqlglot.optimizer.annotate_types import annotate_types
    576
    577                this = annotate_types(this)
    578
    579            if this.is_type(*exp.DataType.TEXT_TYPES):
    580                this = exp.Encode(this=this, charset=exp.Literal.string("utf-8"))
    581
    582            return self.func("LOWER", self.func("TO_HEX", self.func("MD5", self.sql(this))))

def strtounix_sql(self, expression: sqlglot.expressions.StrToUnix) -> str: View Source

    584        def strtounix_sql(self, expression: exp.StrToUnix) -> str:
    585            # Since `TO_UNIXTIME` requires a `TIMESTAMP`, we need to parse the argument into one.
    586            # To do this, we first try to `DATE_PARSE` it, but since this can fail when there's a
    587            # timezone involved, we wrap it in a `TRY` call and use `PARSE_DATETIME` as a fallback,
    588            # which seems to be using the same time mapping as Hive, as per:
    589            # https://joda-time.sourceforge.net/apidocs/org/joda/time/format/DateTimeFormat.html
    590            this = expression.this
    591            value_as_text = exp.cast(this, exp.DataType.Type.TEXT)
    592            value_as_timestamp = (
    593                exp.cast(this, exp.DataType.Type.TIMESTAMP) if this.is_string else this
    594            )
    595
    596            parse_without_tz = self.func("DATE_PARSE", value_as_text, self.format_time(expression))
    597
    598            formatted_value = self.func(
    599                "DATE_FORMAT", value_as_timestamp, self.format_time(expression)
    600            )
    601            parse_with_tz = self.func(
    602                "PARSE_DATETIME",
    603                formatted_value,
    604                self.format_time(expression, Hive.INVERSE_TIME_MAPPING, Hive.INVERSE_TIME_TRIE),
    605            )
    606            coalesced = self.func("COALESCE", self.func("TRY", parse_without_tz), parse_with_tz)
    607            return self.func("TO_UNIXTIME", coalesced)

def bracket_sql(self, expression: sqlglot.expressions.Bracket) -> str: View Source

    609        def bracket_sql(self, expression: exp.Bracket) -> str:
    610            if expression.args.get("safe"):
    611                return self.func(
    612                    "ELEMENT_AT",
    613                    expression.this,
    614                    seq_get(
    615                        apply_index_offset(
    616                            expression.this,
    617                            expression.expressions,
    618                            1 - expression.args.get("offset", 0),
    619                        ),
    620                        0,
    621                    ),
    622                )
    623            return super().bracket_sql(expression)

def struct_sql(self, expression: sqlglot.expressions.Struct) -> str: View Source

    625        def struct_sql(self, expression: exp.Struct) -> str:
    626            from sqlglot.optimizer.annotate_types import annotate_types
    627
    628            expression = annotate_types(expression)
    629            values: t.List[str] = []
    630            schema: t.List[str] = []
    631            unknown_type = False
    632
    633            for e in expression.expressions:
    634                if isinstance(e, exp.PropertyEQ):
    635                    if e.type and e.type.is_type(exp.DataType.Type.UNKNOWN):
    636                        unknown_type = True
    637                    else:
    638                        schema.append(f"{self.sql(e, 'this')} {self.sql(e.type)}")
    639                    values.append(self.sql(e, "expression"))
    640                else:
    641                    values.append(self.sql(e))
    642
    643            size = len(expression.expressions)
    644
    645            if not size or len(schema) != size:
    646                if unknown_type:
    647                    self.unsupported(
    648                        "Cannot convert untyped key-value definitions (try annotate_types)."
    649                    )
    650                return self.func("ROW", *values)
    651            return f"CAST(ROW({', '.join(values)}) AS ROW({', '.join(schema)}))"

def interval_sql(self, expression: sqlglot.expressions.Interval) -> str: View Source

    653        def interval_sql(self, expression: exp.Interval) -> str:
    654            if expression.this and expression.text("unit").upper().startswith("WEEK"):
    655                return f"({expression.this.name} * INTERVAL '7' DAY)"
    656            return super().interval_sql(expression)

def transaction_sql(self, expression: sqlglot.expressions.Transaction) -> str: View Source

    658        def transaction_sql(self, expression: exp.Transaction) -> str:
    659            modes = expression.args.get("modes")
    660            modes = f" {', '.join(modes)}" if modes else ""
    661            return f"START TRANSACTION{modes}"

def offset_limit_modifiers( self, expression: sqlglot.expressions.Expression, fetch: bool, limit: Union[sqlglot.expressions.Fetch, sqlglot.expressions.Limit, NoneType]) -> List[str]: View Source

    663        def offset_limit_modifiers(
    664            self, expression: exp.Expression, fetch: bool, limit: t.Optional[exp.Fetch | exp.Limit]
    665        ) -> t.List[str]:
    666            return [
    667                self.sql(expression, "offset"),
    668                self.sql(limit),
    669            ]

def create_sql(self, expression: sqlglot.expressions.Create) -> str: View Source

    671        def create_sql(self, expression: exp.Create) -> str:
    672            """
    673            Presto doesn't support CREATE VIEW with expressions (ex: `CREATE VIEW x (cola)` then `(cola)` is the expression),
    674            so we need to remove them
    675            """
    676            kind = expression.args["kind"]
    677            schema = expression.this
    678            if kind == "VIEW" and schema.expressions:
    679                expression.this.set("expressions", None)
    680            return super().create_sql(expression)

Presto doesn't support CREATE VIEW with expressions (ex: `CREATE VIEW x (cola)` then `(cola)` is the expression), so we need to remove them

def delete_sql(self, expression: sqlglot.expressions.Delete) -> str: View Source

    682        def delete_sql(self, expression: exp.Delete) -> str:
    683            """
    684            Presto only supports DELETE FROM for a single table without an alias, so we need
    685            to remove the unnecessary parts. If the original DELETE statement contains more
    686            than one table to be deleted, we can't safely map it 1-1 to a Presto statement.
    687            """
    688            tables = expression.args.get("tables") or [expression.this]
    689            if len(tables) > 1:
    690                return super().delete_sql(expression)
    691
    692            table = tables[0]
    693            expression.set("this", table)
    694            expression.set("tables", None)
    695
    696            if isinstance(table, exp.Table):
    697                table_alias = table.args.get("alias")
    698                if table_alias:
    699                    table_alias.pop()
    700                    expression = t.cast(exp.Delete, expression.transform(unqualify_columns))
    701
    702            return super().delete_sql(expression)

Presto only supports DELETE FROM for a single table without an alias, so we need to remove the unnecessary parts. If the original DELETE statement contains more than one table to be deleted, we can't safely map it 1-1 to a Presto statement.

SELECT_KINDS: Tuple[str, ...] = ()
AFTER_HAVING_MODIFIER_TRANSFORMS = {'windows': <function Generator.<lambda>>, 'qualify': <function Generator.<lambda>>}

##### Inherited Members

sqlglot.generator.Generator
  Generator
  NULL_ORDERING_SUPPORTED
  IGNORE_NULLS_IN_FUNC
  LOCKING_READS_SUPPORTED
  EXPLICIT_SET_OP
  WRAP_DERIVED_VALUES
  CREATE_FUNCTION_RETURN_AS
  MATCHED_BY_SOURCE
  SINGLE_STRING_INTERVAL
  LIMIT_FETCH
  RENAME_TABLE_WITH_DB
  GROUPINGS_SEP
  INDEX_ON
  QUERY_HINT_SEP
  DUPLICATE_KEY_UPDATE_WITH_SET
  LIMIT_IS_TOP
  RETURNING_END
  EXTRACT_ALLOWS_QUOTES
  VALUES_AS_TABLE
  ALTER_TABLE_INCLUDE_COLUMN_KEYWORD
  UNNEST_WITH_ORDINALITY
  AGGREGATE_FILTER_SUPPORTED
  SEMI_ANTI_JOIN_WITH_SIDE
  COMPUTED_COLUMN_WITH_TYPE
  SUPPORTS_TABLE_COPY
  TABLESAMPLE_REQUIRES_PARENS
  TABLESAMPLE_SIZE_IS_ROWS
  TABLESAMPLE_KEYWORDS
  TABLESAMPLE_WITH_METHOD
  TABLESAMPLE_SEED_KEYWORD
  COLLATE_IS_FUNC
  DATA_TYPE_SPECIFIERS_ALLOWED
  ENSURE_BOOLS
  CTE_RECURSIVE_KEYWORD_REQUIRED
  LAST_DAY_SUPPORTS_DATE_PART
  SUPPORTS_TABLE_ALIAS_COLUMNS
  UNPIVOT_ALIASES_ARE_IDENTIFIERS
  JSON_KEY_VALUE_PAIR_SEP
  INSERT_OVERWRITE
  SUPPORTS_SELECT_INTO
  SUPPORTS_UNLOGGED_TABLES
  SUPPORTS_CREATE_TABLE_LIKE
  JSON_TYPE_REQUIRED_FOR_EXTRACTION
  JSON_PATH_BRACKETED_KEY_SUPPORTED
  JSON_PATH_SINGLE_QUOTE_ESCAPE
  SUPPORTED_JSON_PATH_PARTS
  CAN_IMPLEMENT_ARRAY_ANY
  SET_OP_MODIFIERS
  COPY_PARAMS_ARE_WRAPPED
  COPY_PARAMS_EQ_REQUIRED
  COPY_HAS_INTO_KEYWORD
  TRY_SUPPORTED
  SUPPORTS_UESCAPE
  STAR_EXCEPT
  WITH_PROPERTIES_PREFIX
  QUOTE_JSON_PATH
  TIME_PART_SINGULARS
  TOKEN_MAPPING
  PARAMETER_TOKEN
  NAMED_PLACEHOLDER_TOKEN
  WITH_SEPARATED_COMMENTS
  EXCLUDE_COMMENTS
  UNWRAPPED_INTERVAL_VALUES
  PARAMETERIZABLE_TEXT_TYPES
  EXPRESSIONS_WITHOUT_NESTED_CTES
  SENTINEL_LINE_BREAK
  pretty
  identify
  normalize
  pad
  unsupported_level
  max_unsupported
  leading_comma
  max_text_width
  comments
  dialect
  normalize_functions
  unsupported_messages
  generate
  preprocess
  unsupported
  sep
  seg
  pad_comment
  maybe_comment
  wrap
  no_identify
  normalize_func
  indent
  sql
  uncache_sql
  cache_sql
  characterset_sql
  column_parts
  column_sql
  columnposition_sql
  columndef_sql
  columnconstraint_sql
  computedcolumnconstraint_sql
  autoincrementcolumnconstraint_sql
  compresscolumnconstraint_sql
  generatedasidentitycolumnconstraint_sql
  generatedasrowcolumnconstraint_sql
  periodforsystemtimeconstraint_sql
  notnullcolumnconstraint_sql
  transformcolumnconstraint_sql
  primarykeycolumnconstraint_sql
  uniquecolumnconstraint_sql
  createable_sql
  sequenceproperties_sql
  clone_sql
  describe_sql
  heredoc_sql
  prepend_ctes
  with_sql
  cte_sql
  tablealias_sql
  bitstring_sql
  hexstring_sql
  bytestring_sql
  unicodestring_sql
  rawstring_sql
  datatypeparam_sql
  datatype_sql
  directory_sql
  drop_sql
  except_sql
  except_op
  fetch_sql
  filter_sql
  hint_sql
  indexparameters_sql
  index_sql
  identifier_sql
  hex_sql
  lowerhex_sql
  inputoutputformat_sql
  national_sql
  partition_sql
  properties_sql
  root_properties
  properties
  with_properties
  locate_properties
  property_name
  property_sql
  likeproperty_sql
  fallbackproperty_sql
  journalproperty_sql
  freespaceproperty_sql
  checksumproperty_sql
  mergeblockratioproperty_sql
  datablocksizeproperty_sql
  blockcompressionproperty_sql
  isolatedloadingproperty_sql
  partitionboundspec_sql
  partitionedofproperty_sql
  lockingproperty_sql
  withdataproperty_sql
  withsystemversioningproperty_sql
  insert_sql
  intersect_sql
  intersect_op
  introducer_sql
  kill_sql
  pseudotype_sql
  objectidentifier_sql
  onconflict_sql
  returning_sql
  rowformatdelimitedproperty_sql
  withtablehint_sql
  indextablehint_sql
  historicaldata_sql
  table_parts
  table_sql
  tablesample_sql
  pivot_sql
  version_sql
  tuple_sql
  update_sql
  values_sql
  var_sql
  into_sql
  from_sql
  group_sql
  having_sql
  connect_sql
  prior_sql
  join_sql
  lambda_sql
  lateral_op
  lateral_sql
  limit_sql
  offset_sql
  setitem_sql
  set_sql
  pragma_sql
  lock_sql
  literal_sql
  escape_str
  loaddata_sql
  null_sql
  boolean_sql
  order_sql
  withfill_sql
  cluster_sql
  distribute_sql
  sort_sql
  ordered_sql
  matchrecognizemeasure_sql
  matchrecognize_sql
  query_modifiers
  options_modifier
  queryoption_sql
  after_limit_modifiers
  select_sql
  schema_sql
  schema_columns_sql
  star_sql
  parameter_sql
  sessionparameter_sql
  placeholder_sql
  subquery_sql
  qualify_sql
  set_operations
  union_sql
  union_op
  unnest_sql
  prewhere_sql
  where_sql
  window_sql
  partition_by_sql
  windowspec_sql
  withingroup_sql
  between_sql
  bracket_offset_expressions
  all_sql
  any_sql
  exists_sql
  case_sql
  constraint_sql
  nextvaluefor_sql
  extract_sql
  trim_sql
  convert_concat_args
  concat_sql
  concatws_sql
  check_sql
  foreignkey_sql
  primarykey_sql
  if_sql
  matchagainst_sql
  jsonkeyvalue_sql
  jsonpath_sql
  json_path_part
  formatjson_sql
  jsonobject_sql
  jsonobjectagg_sql
  jsonarray_sql
  jsonarrayagg_sql
  jsoncolumndef_sql
  jsonschema_sql
  jsontable_sql
  openjsoncolumndef_sql
  openjson_sql
  in_sql
  in_unnest_op
  return_sql
  reference_sql
  anonymous_sql
  paren_sql
  neg_sql
  not_sql
  alias_sql
  pivotalias_sql
  aliases_sql
  atindex_sql
  attimezone_sql
  fromtimezone_sql
  add_sql
  and_sql
  or_sql
  xor_sql
  connector_sql
  bitwiseand_sql
  bitwiseleftshift_sql
  bitwisenot_sql
  bitwiseor_sql
  bitwiserightshift_sql
  bitwisexor_sql
  cast_sql
  currentdate_sql
  collate_sql
  command_sql
  comment_sql
  mergetreettlaction_sql
  mergetreettl_sql
  commit_sql
  rollback_sql
  altercolumn_sql
  alterdiststyle_sql
  altersortkey_sql
  renametable_sql
  renamecolumn_sql
  alterset_sql
  altertable_sql
  add_column_sql
  droppartition_sql
  addconstraint_sql
  distinct_sql
  ignorenulls_sql
  respectnulls_sql
  havingmax_sql
  intdiv_sql
  dpipe_sql
  div_sql
  overlaps_sql
  distance_sql
  dot_sql
  eq_sql
  propertyeq_sql
  escape_sql
  glob_sql
  gt_sql
  gte_sql
  ilike_sql
  ilikeany_sql
  is_sql
  like_sql
  likeany_sql
  similarto_sql
  lt_sql
  lte_sql
  mod_sql
  mul_sql
  neq_sql
  nullsafeeq_sql
  nullsafeneq_sql
  slice_sql
  sub_sql
  trycast_sql
  try_sql
  log_sql
  use_sql
  binary
  function_fallback_sql
  func
  format_args
  too_wide
  format_time
  expressions
  op_expressions
  naked_property
  tag_sql
  token_sql
  userdefinedfunction_sql
  joinhint_sql
  kwarg_sql
  when_sql
  merge_sql
  tochar_sql
  tonumber_sql
  dictproperty_sql
  dictrange_sql
  dictsubproperty_sql
  oncluster_sql
  clusteredbyproperty_sql
  anyvalue_sql
  querytransform_sql
  indexconstraintoption_sql
  checkcolumnconstraint_sql
  indexcolumnconstraint_sql
  nvl2_sql
  comprehension_sql
  columnprefix_sql
  opclass_sql
  predict_sql
  forin_sql
  refresh_sql
  operator_sql
  toarray_sql
  tsordstotime_sql
  tsordstotimestamp_sql
  tsordstodate_sql
  unixdate_sql
  lastday_sql
  dateadd_sql
  arrayany_sql
  partitionrange_sql
  truncatetable_sql
  convert_sql
  copyparameter_sql
  credentials_sql
  copy_sql
  semicolon_sql
  datadeletionproperty_sql
  maskingpolicycolumnconstraint_sql
  gapfill_sql
  scope_resolution
  scoperesolution_sql
  parsejson_sql
  rand_sql
  changes_sql
  pad_sql
  summarize_sql